import random

import abjad

import auxjad


def test_example_of_usage_03():
    random.seed(12432)
    pitch_selector = auxjad.TenneySelector([0, 7, 8, 2, 3, 10])
    duration_selector = auxjad.CartographySelector([(2, 8), (3, 8), (5, 8)])
    dynamic_selector = auxjad.CartographySelector(['p', 'mp', 'mf', 'f'])
    articulation_selector = auxjad.CartographySelector([None, '-', '>'])
    pitches = [pitch_selector() for _ in range(8)]
    durations = [duration_selector() for _ in range(8)]
    dynamics = [dynamic_selector() for _ in range(8)]
    articulations = [articulation_selector() for _ in range(8)]
    leaf_dyn_maker = auxjad.LeafDynMaker()
    notes = leaf_dyn_maker(pitches, durations, dynamics, articulations)
    container = abjad.Container(notes)
    looper = auxjad.WindowLooper(container,
                                 window_size=(5, 4),
                                 step_size=(1, 4),
                                 )
    staff = abjad.Staff()
    notes = looper.output_n(6)
    staff.append(notes)
    looper.step_size = (1, 16)
    notes = looper.output_n(6)
    staff.append(notes)
    abjad.mutate.remove_repeated_time_signatures(staff[:])
    looper.window_size = (3, 4)
    notes = looper.output_n(6)
    staff.append(notes)
    abjad.mutate.remove_repeated_dynamics(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 5/4
            bf'4.
            \f
            g'4.
            af'4
            \mf
            c'4
            - \tenuto
            bf'8
            \f
            g'4.
            af'4
            \mf
            c'4.
            - \tenuto
            g'8
            \f
            g'4
            af'4
            \mf
            c'4
            - \tenuto
            ~
            c'8
            g'4
            \f
            ef'8
            \p
            af'4
            \mf
            c'4.
            - \tenuto
            g'8
            \f
            ~
            g'8
            ef'4.
            \p
            c'4.
            \mf
            - \tenuto
            g'4
            \f
            ef'8
            \p
            ~
            ef'4
            bf'4
            c'8
            \mf
            - \tenuto
            g'4
            \f
            ef'4.
            \p
            bf'2
            c'16
            \mf
            - \tenuto
            g'8.
            \f
            ~
            g'16
            ef'8.
            \p
            ~
            ef'8.
            bf'16
            ~
            bf'4..
            ~
            bf'16
            g'4
            \f
            ef'4.
            \p
            bf'8
            ~
            bf'4.
            ~
            bf'8
            g'8.
            \f
            ef'16
            \p
            ~
            ef'4
            ~
            ef'16
            bf'8.
            ~
            bf'4
            ~
            bf'16
            ~
            bf'8
            af'16
            - \tenuto
            g'8
            \f
            ef'4.
            \p
            bf'4
            ~
            bf'4
            ~
            bf'8
            af'8
            - \tenuto
            g'16
            \f
            ef'8.
            \p
            ~
            ef'8.
            bf'16
            ~
            bf'4
            ~
            bf'4
            ~
            bf'16
            af'8.
            - \tenuto
            ef'4.
            bf'4.
            ~
            bf'4
            af'4
            - \tenuto
            \time 3/4
            ef'4
            ~
            ef'16
            bf'4..
            ef'4
            bf'2
            ef'8.
            bf'16
            ~
            bf'2
            ef'8
            bf'8
            ~
            bf'2
            ef'16
            bf'8.
            ~
            bf'4..
            af'16
            - \tenuto
            bf'2
            ~
            bf'8
            af'8
            - \tenuto
        }
        """
    )



import abjad

import auxjad


def test_respell_augmented_unisons_01():
    staff = abjad.Staff(r"c'4 r4 <ef' e'>4 g'4 <c' cs'>4 r2.")
    auxjad.mutate.respell_augmented_unisons(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'4
            r4
            <ds' e'>4
            g'4
            <c' df'>4
            r2.
        }
        """
    )


def test_respell_augmented_unisons_02():
    staff = abjad.Staff()
    for pitch in range(12):
        staff.append(abjad.Chord([pitch, pitch + 1], (1, 16)))
    auxjad.mutate.respell_augmented_unisons(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            <c' df'>16
            <cs' d'>16
            <d' ef'>16
            <ds' e'>16
            <e' f'>16
            <f' gf'>16
            <fs' g'>16
            <g' af'>16
            <gs' a'>16
            <a' bf'>16
            <as' b'>16
            <b' c''>16
        }
        """
    )


def test_respell_augmented_unisons_03():
    staff = abjad.Staff(r"<a c' cs' f'>1")
    auxjad.mutate.respell_augmented_unisons(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            <a c' df' f'>1
        }
        """
    )


def test_respell_augmented_unisons_04():
    staff = abjad.Staff(r"<e' cs' g' ef'>1")
    auxjad.mutate.respell_augmented_unisons(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            <cs' ds' e' g'>1
        }
        """
    )


def test_respell_augmented_unisons_05():
    staff = abjad.Staff(r"<c' cs''>1")
    auxjad.mutate.respell_augmented_unisons(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            <c' cs''>1
        }
        """
    )
    auxjad.mutate.respell_augmented_unisons(staff[:], include_multiples=True)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            <c' df''>1
        }
        """
    )


def test_respell_augmented_unisons_06():
    staff = abjad.Staff(r"<c' cs' cs''>1")
    auxjad.mutate.respell_augmented_unisons(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            <c' df' cs''>1
        }
        """
    )
    staff = abjad.Staff(r"<c' cs' cs''>1")
    auxjad.mutate.respell_augmented_unisons(
        staff[:],
        respell_by_pitch_class=True,
    )
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            <c' df' df''>1
        }
        """
    )


def test_respell_augmented_unisons_07():
    staff = abjad.Staff(
        r"""
        <e' bs'>1
        <e' b' bs'>1
        <e' cf''>1
        <e' cf'' c''>1
        <es' c''>1
        <e' es' c''>1
        <ff' c''>1
        <ff' f' c''>1
        """
    )
    auxjad.mutate.respell_augmented_unisons(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            <e' bs'>1
            <e' b' c''>1
            <e' cf''>1
            <e' b' c''>1
            <es' c''>1
            <e' f' c''>1
            <ff' c''>1
            <e' f' c''>1
        }
        """
    )


def test_respell_augmented_unisons_08():
    staff = abjad.Staff(r"c'4 r4 <ef' e'>4 g'4 <c' cs'>4 r2.")
    abjad.mutate.respell_augmented_unisons(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'4
            r4
            <ds' e'>4
            g'4
            <c' df'>4
            r2.
        }
        """
    )



import abjad

import auxjad


def test_remove_repeated_dynamics_01():
    staff = abjad.Staff(r"c'4\pp d'8\pp | c'4\f d'8\f")
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'4
            \pp
            d'8
            \pp
            c'4
            \f
            d'8
            \f
        }
        """
    )
    auxjad.mutate.remove_repeated_dynamics(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'4
            \pp
            d'8
            c'4
            \f
            d'8
        }
        """
    )


def test_remove_repeated_dynamics_02():
    staff = abjad.Staff(r"c'4\p d'8 | e'4.\p | c'4\p d'8\f")
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'4
            \p
            d'8
            e'4.
            \p
            c'4
            \p
            d'8
            \f
        }
        """
    )
    auxjad.mutate.remove_repeated_dynamics(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'4
            \p
            d'8
            e'4.
            c'4
            d'8
            \f
        }
        """
    )


def test_remove_repeated_dynamics_03():
    staff = abjad.Staff([abjad.Note("c'2"),
                         abjad.Chord("<d' f'>2"),
                         abjad.Tuplet((2, 3), "g2 a2 b2"),
                         ])
    abjad.attach(abjad.Dynamic('ppp'), staff[0])
    abjad.attach(abjad.Dynamic('ppp'), staff[1])
    abjad.attach(abjad.Dynamic('ppp'), staff[2][0])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'2
            \ppp
            <d' f'>2
            \ppp
            \times 2/3
            {
                g2
                \ppp
                a2
                b2
            }
        }
        """
    )
    auxjad.mutate.remove_repeated_dynamics(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'2
            \ppp
            <d' f'>2
            \times 2/3
            {
                g2
                a2
                b2
            }
        }
        """
    )


def test_remove_repeated_dynamics_04():
    staff = abjad.Staff(r"c'4\pp\< d'8\f\> | c'4\f d'8\f")
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'4
            \pp
            \<
            d'8
            \f
            \>
            c'4
            \f
            d'8
            \f
        }
        """
    )
    auxjad.mutate.remove_repeated_dynamics(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'4
            \pp
            \<
            d'8
            \f
            \>
            c'4
            \f
            d'8
        }
        """
    )


def test_remove_repeated_dynamics_05():
    staff = abjad.Staff(r"c'4\pp\< d'8\f\> | c'4\f d'8\f")
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'4
            \pp
            \<
            d'8
            \f
            \>
            c'4
            \f
            d'8
            \f
        }
        """
    )
    auxjad.mutate.remove_repeated_dynamics(staff[:], ignore_hairpins=True)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'4
            \pp
            \<
            d'8
            \f
            \>
            c'4
            d'8
        }
        """
    )


def test_remove_repeated_dynamics_06():
    staff = abjad.Staff(r"c'4\pp r2. | c'1\pp")
    auxjad.mutate.remove_repeated_dynamics(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'4
            \pp
            r2.
            c'1
        }
        """
    )
    staff = abjad.Staff(r"c'4\pp r2. | c'1\pp")
    auxjad.mutate.remove_repeated_dynamics(staff[:], reset_after_rests=True)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'4
            \pp
            r2.
            c'1
            \pp
        }
        """
    )


def test_remove_repeated_dynamics_07():
    staff = abjad.Staff(r"c'4\pp r2. | c'1\pp")
    auxjad.mutate.remove_repeated_dynamics(staff[:],
                                           reset_after_rests=True,
                                           reset_after_duration=(4, 4),
                                           )
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'4
            \pp
            r2.
            c'1
        }
        """
    )
    staff = abjad.Staff(r"c'4\pp r2. | c'1\pp")
    auxjad.mutate.remove_repeated_dynamics(staff[:],
                                           reset_after_rests=True,
                                           reset_after_duration=2 / 4,
                                           )
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'4
            \pp
            r2.
            c'1
            \pp
        }
        """
    )


def test_remove_repeated_dynamics_08():
    staff = abjad.Staff(r"c'4\pp r2. | R1 | c'1\pp")
    auxjad.mutate.remove_repeated_dynamics(
        staff[:],
        reset_after_rests=True,
        reset_after_duration=abjad.Duration(4, 4),
    )
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'4
            \pp
            r2.
            R1
            c'1
            \pp
        }
        """
    )
    staff = abjad.Staff(r"c'4\pp r2. | R1 | c'1\pp")
    auxjad.mutate.remove_repeated_dynamics(staff[:],
                                           reset_after_rests=True,
                                           reset_after_duration=2,
                                           )
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'4
            \pp
            r2.
            R1
            c'1
        }
        """
    )


def test_remove_repeated_dynamics_09():
    staff = abjad.Staff(r"c'4\pp d'8\pp | c'4\f d'8\f")
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'4
            \pp
            d'8
            \pp
            c'4
            \f
            d'8
            \f
        }
        """
    )
    abjad.mutate.remove_repeated_dynamics(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'4
            \pp
            d'8
            c'4
            \f
            d'8
        }
        """
    )


def test_remove_repeated_dynamics_10():
    staff1 = abjad.Staff(r"c'4\pp\> d' e' f'\! c'\pp\> d' e' f'\! g'1\pp")
    staff2 = abjad.Staff(r"c'4\pp d'\> e' f'\! c'\pp d'\> e' f'\! g'1\pp")
    abjad.mutate.remove_repeated_dynamics(staff1[:])
    abjad.mutate.remove_repeated_dynamics(staff2[:])
    assert abjad.lilypond(staff1) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'4
            \pp
            \>
            d'4
            e'4
            f'4
            \!
            c'4
            \pp
            \>
            d'4
            e'4
            f'4
            \!
            g'1
            \pp
        }
        """
    )
    assert abjad.lilypond(staff2) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'4
            \pp
            d'4
            \>
            e'4
            f'4
            \!
            c'4
            \pp
            d'4
            \>
            e'4
            f'4
            \!
            g'1
            \pp
        }

        """
    )



import abjad

import auxjad


def test_reposition_slurs_01():
    staff = abjad.Staff(r"c'1( d'2 r2) r1 e'1")
    auxjad.mutate.reposition_slurs(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            (
            d'2
            )
            r2
            r1
            e'1
        }
        """
    )


def test_reposition_slurs_02():
    staff = abjad.Staff(r"c'1 r2( d'2 e'1)")
    auxjad.mutate.reposition_slurs(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            r2
            d'2
            (
            e'1
            )
        }
        """
    )


def test_reposition_slurs_03():
    staff = abjad.Staff(r"c'1( d'2 r2 r1) e'1")
    auxjad.mutate.reposition_slurs(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            (
            d'2
            )
            r2
            r1
            e'1
        }
        """
    )


def test_reposition_slurs_04():
    staff = abjad.Staff(r"c'1 r2( r1 d'2 e'1)")
    auxjad.mutate.reposition_slurs(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            r2
            r1
            d'2
            (
            e'1
            )
        }
        """
    )


def test_reposition_slurs_05():
    staff = abjad.Staff(r"c'1( d'2 e'2 f'1)")
    auxjad.mutate.reposition_slurs(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            (
            d'2
            e'2
            f'1
            )
        }
        """
    )


def test_reposition_slurs_06():
    staff = abjad.Staff(r"c'1( d'2 r2 e'1 f'1)")
    auxjad.mutate.reposition_slurs(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            (
            d'2
            )
            r2
            e'1
            (
            f'1
            )
        }
        """
    )


def test_reposition_slurs_07():
    staff = abjad.Staff(r"c'1( d'2 r2 e'1 f'1)")
    auxjad.mutate.reposition_slurs(staff[:], allow_slurs_under_rests=True)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            (
            d'2
            r2
            e'1
            f'1
            )
        }
        """
    )


def test_reposition_slurs_08():
    staff = abjad.Staff(r"r1( c'1 d'2 r2 e'1 f'1)")
    auxjad.mutate.reposition_slurs(staff[:], allow_slurs_under_rests=True)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            r1
            c'1
            (
            d'2
            r2
            e'1
            f'1
            )
        }
        """
    )


def test_reposition_slurs_09():
    staff = abjad.Staff(r"c'1( r2 d'2 e'1)")
    auxjad.mutate.reposition_slurs(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            r2
            d'2
            (
            e'1
            )
        }
        """
    )


def test_reposition_slurs_10():
    staff = abjad.Staff(r"c'1( d'2 r2 e'1)")
    auxjad.mutate.reposition_slurs(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            (
            d'2
            )
            r2
            e'1
        }
        """
    )


def test_reposition_slurs_11():
    staff = abjad.Staff(r"c'1( d'2 r2 e'2 f'2) g'1( a'1")
    auxjad.mutate.reposition_slurs(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            (
            d'2
            )
            r2
            e'2
            (
            f'2
            )
            g'1
            (
            a'1
            )
        }
        """
    )
    staff = abjad.Staff(r"c'1( d'2 r2 e'2 f'2) g'1( a'1")
    auxjad.mutate.reposition_slurs(
        staff[:],
        close_unterminated_final_slur=False,
    )
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            (
            d'2
            )
            r2
            e'2
            (
            f'2
            )
            g'1
            (
            a'1
        }
        """
    )
    staff = abjad.Staff(r"c'1( d'2 r2 e'2 f'2) g'1( r1")
    auxjad.mutate.reposition_slurs(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            (
            d'2
            )
            r2
            e'2
            (
            f'2
            )
            g'1
            r1
        }
        """
    )
    staff = abjad.Staff(r"c'1( d'2 r2 e'2 f'2) g'1(")
    auxjad.mutate.reposition_slurs(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            (
            d'2
            )
            r2
            e'2
            (
            f'2
            )
            g'1
        }
        """
    )


def test_reposition_slurs_12():
    staff = abjad.Staff(r"c'1( d'2) e'2) f'2( g'2( a'1)")
    auxjad.mutate.reposition_slurs(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            (
            d'2
            )
            e'2
            f'2
            (
            g'2
            a'1
            )
        }
        """
    )


def test_reposition_slurs_13():
    staff = abjad.Staff(r"c'1( d'2 r2) r1 e'1")
    abjad.mutate.reposition_slurs(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            (
            d'2
            )
            r2
            r1
            e'1
        }
        """
    )


def test_reposition_slurs_14():
    staff = abjad.Staff(r"c'1( d'1 e'1 f'1")
    auxjad.mutate.reposition_slurs(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            (
            d'1
            e'1
            f'1
            )
        }
        """
    )
    staff = abjad.Staff(r"c'1( d'1( e'1 f'1)")
    auxjad.mutate.reposition_slurs(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            (
            d'1
            e'1
            f'1
            )
        }
        """
    )
    staff = abjad.Staff(r"c'1( d'1 e'1) f'1")
    auxjad.mutate.reposition_slurs(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            (
            d'1
            e'1
            )
            f'1
        }
        """
    )
    staff = abjad.Staff(r"c'1( d'1 e'1) f'1)")
    auxjad.mutate.reposition_slurs(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            (
            d'1
            e'1
            )
            f'1
        }
        """
    )


def test_reposition_slurs_15():
    staff = abjad.Staff(r"c'1( d'1 r1 r1 r1 e'1 f'1)")
    auxjad.mutate.reposition_slurs(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            (
            d'1
            )
            r1
            r1
            r1
            e'1
            (
            f'1
            )
        }
        """
    )
    staff = abjad.Staff(r"c'1( d'1 r1 r1 r1 e'1 f'1)")
    auxjad.mutate.reposition_slurs(staff[:],
                                   allow_slurs_under_rests=True,
                                   )
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            (
            d'1
            r1
            r1
            r1
            e'1
            f'1
            )
        }
        """
    )



import abjad

import auxjad


def test_prettify_rewrite_meter_01():
    staff = abjad.Staff(
        r"\time 3/4 c'16 d'8 e'16 f'16 g'16 a'8 b'8 c''16 d''16"
    )
    meter = abjad.Meter((3, 4))
    abjad.Meter.rewrite_meter(staff[:], meter)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            c'16
            d'16
            ~
            d'16
            e'16
            f'16
            g'16
            a'8
            b'8
            c''16
            d''16
        }
        """
    )
    auxjad.mutate.prettify_rewrite_meter(staff[:], meter)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            c'16
            d'8
            e'16
            f'16
            g'16
            a'8
            b'8
            c''16
            d''16
        }
        """
    )


def test_prettify_rewrite_meter_02():
    staff = abjad.Staff(
        r"\time 3/4 c'32 d'32 e'8 f'16 "
        r"\times 2/3 {g'32 a'32 b'32} c''8 "
        r"r16 r32. d''64 e''8 f''32 g''32"
    )
    meter = abjad.Meter((3, 4))
    abjad.Meter.rewrite_meter(staff[:], meter)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            c'32
            d'32
            e'16
            ~
            e'16
            f'16
            \times 2/3
            {
                g'32
                a'32
                b'32
            }
            c''16
            ~
            c''16
            r16
            r32.
            d''64
            e''16
            ~
            e''16
            f''32
            g''32
        }
        """
    )
    auxjad.mutate.prettify_rewrite_meter(staff[:], meter)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            c'32
            d'32
            e'8
            f'16
            \times 2/3
            {
                g'32
                a'32
                b'32
            }
            c''8
            r16
            r32.
            d''64
            e''8
            f''32
            g''32
        }
        """
    )


def test_prettify_rewrite_meter_03():
    staff = abjad.Staff(r"\time 6/4 c'8 d'4 e'4 f'4 g'4 a'4 b'8")
    meter = abjad.Meter((6, 4))
    abjad.Meter.rewrite_meter(staff[:], meter)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 6/4
            c'8
            d'8
            ~
            d'8
            e'8
            ~
            e'8
            f'8
            ~
            f'8
            g'8
            ~
            g'8
            a'8
            ~
            a'8
            b'8
        }
        """
    )
    auxjad.mutate.prettify_rewrite_meter(staff[:], meter)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 6/4
            c'8
            d'4
            e'4
            f'8
            ~
            f'8
            g'4
            a'4
            b'8
        }
        """
    )


def test_prettify_rewrite_meter_04():
    staff = abjad.Staff(r"\time 6/4 c'8 d'4 e'4 f'4 g'4 a'4 b'8")
    meter = abjad.Meter((6, 4))
    abjad.Meter.rewrite_meter(staff[:], meter)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 6/4
            c'8
            d'8
            ~
            d'8
            e'8
            ~
            e'8
            f'8
            ~
            f'8
            g'8
            ~
            g'8
            a'8
            ~
            a'8
            b'8
        }
        """
    )
    auxjad.mutate.prettify_rewrite_meter(
        staff[:],
        meter,
        fuse_across_groups_of_beats=False,
    )
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 6/4
            c'8
            d'8
            ~
            d'8
            e'8
            ~
            e'8
            f'8
            ~
            f'8
            g'8
            ~
            g'8
            a'8
            ~
            a'8
            b'8
        }
        """
    )


def test_prettify_rewrite_meter_05():
    staff = abjad.Staff(r"\time 7/4 c'8 d'4 e'4 f'4 g'4 a'4 b'4 c''8")
    meter = abjad.Meter((7, 4))
    abjad.Meter.rewrite_meter(staff[:], meter)
    auxjad.mutate.prettify_rewrite_meter(staff[:], meter)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 7/4
            c'8
            d'4
            e'4
            f'8
            ~
            f'8
            g'4
            a'8
            ~
            a'8
            b'4
            c''8
        }
        """
    )
    staff = abjad.Staff(r"\time 7/4 c'8 d'4 e'4 f'4 g'4 a'4 b'4 c''8")
    meter = abjad.Meter((7, 4), increase_monotonic=True)
    abjad.Meter.rewrite_meter(staff[:], meter)
    auxjad.mutate.prettify_rewrite_meter(staff[:], meter)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 7/4
            c'8
            d'4
            e'8
            ~
            e'8
            f'4
            g'8
            ~
            g'8
            a'4
            b'4
            c''8
        }
        """
    )


def test_prettify_rewrite_meter_06():
    staff = abjad.Staff(
        r"\time 5/8 c'16 d'8 e'8 f'8 g'8 a'16 ~ "
        r"a'16 b'8 c''8 d''8 e''8 f''16"
    )
    meter = abjad.Meter((5, 8))
    for measure in abjad.select(staff[:]).group_by_measure():
        abjad.Meter.rewrite_meter(measure, meter)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 5/8
            c'16
            d'16
            ~
            d'16
            e'16
            ~
            e'16
            f'16
            ~
            f'16
            g'16
            ~
            g'16
            a'16
            ~
            a'16
            b'16
            ~
            b'16
            c''16
            ~
            c''16
            d''16
            ~
            d''16
            e''16
            ~
            e''16
            f''16
        }
        """
    )
    auxjad.mutate.prettify_rewrite_meter(staff[:], meter)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 5/8
            c'16
            d'8
            e'8
            f'16
            ~
            f'16
            g'8
            a'16
            ~
            a'16
            b'8
            c''8
            d''16
            ~
            d''16
            e''8
            f''16
        }
        """
    )


def test_prettify_rewrite_meter_07():
    staff = abjad.Staff(r"\time 4/4 c'8 d'4 e'4 f'4 g'8")
    meter = abjad.Meter((4, 4))
    abjad.Meter.rewrite_meter(staff[:], meter)
    auxjad.mutate.prettify_rewrite_meter(staff[:], meter)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'8
            d'4
            e'8
            ~
            e'8
            f'4
            g'8
        }
        """
    )
    staff = abjad.Staff(r"\time 4/4 c'8 d'4 e'4 f'4 g'8")
    meter = abjad.Meter((4, 4))
    abjad.Meter.rewrite_meter(staff[:], meter)
    auxjad.mutate.prettify_rewrite_meter(staff[:],
                                         meter,
                                         fuse_quadruple_meter=False,
                                         )
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'8
            d'8
            ~
            d'8
            e'8
            ~
            e'8
            f'8
            ~
            f'8
            g'8
        }
        """
    )


def test_prettify_rewrite_meter_08():
    staff = abjad.Staff(r"\time 3/4 c'8 d'4 e'4 f'8")
    meter = abjad.Meter((3, 4))
    abjad.Meter.rewrite_meter(staff[:], meter)
    auxjad.mutate.prettify_rewrite_meter(staff[:], meter)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            c'8
            d'4
            e'4
            f'8
        }
        """
    )
    staff = abjad.Staff(r"\time 3/4 c'8 d'4 e'4 f'8")
    meter = abjad.Meter((3, 4))
    abjad.Meter.rewrite_meter(staff[:], meter)
    auxjad.mutate.prettify_rewrite_meter(staff[:],
                                         meter,
                                         fuse_triple_meter=False,
                                         )
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            c'8
            d'8
            ~
            d'8
            e'8
            ~
            e'8
            f'8
        }
        """
    )


def test_prettify_rewrite_meter_09():
    staff = abjad.Staff(
        r"\times 2/3 {c'16 d'8 ~ } "
        r"\times 2/3 {d'32 e'8 f'32 ~ } "
        r"f'32 \times 2/3 {g'16 a'32} r32"
    )
    abjad.attach(abjad.TimeSignature((3, 8)), staff[0][0])
    meter = abjad.Meter((3, 8))
    abjad.Meter.rewrite_meter(staff[:], meter)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \times 2/3
            {
                \time 3/8
                c'16
                d'8
                ~
            }
            \times 2/3
            {
                d'32
                e'32
                ~
                e'16.
                f'32
                ~
            }
            f'32
            \times 2/3
            {
                g'16
                a'32
            }
            r32
        }
        """
    )
    auxjad.mutate.prettify_rewrite_meter(staff[:], meter)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \times 2/3
            {
                \time 3/8
                c'16
                d'8
                ~
            }
            \times 2/3
            {
                d'32
                e'32
                ~
                e'16.
                f'32
                ~
            }
            f'32
            \times 2/3
            {
                g'16
                a'32
            }
            r32
        }
        """
    )


def test_prettify_rewrite_meter_10():
    staff = abjad.Staff(
        r"\time 4/4 c'8 d'4 e'4 f'4 g'8 | "
        r"a'8 b'4 c''8 d''16 e''4 f''8."
    )
    meter = abjad.Meter((4, 4))
    for measure in abjad.select(staff[:]).group_by_measure():
        abjad.Meter.rewrite_meter(measure, meter)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'8
            d'8
            ~
            d'8
            e'8
            ~
            e'8
            f'8
            ~
            f'8
            g'8
            a'8
            b'8
            ~
            b'8
            c''8
            d''16
            e''8.
            ~
            e''16
            f''8.
        }
        """
    )
    for measure in abjad.select(staff[:]).group_by_measure():
        auxjad.mutate.prettify_rewrite_meter(measure, meter)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'8
            d'4
            e'8
            ~
            e'8
            f'4
            g'8
            a'8
            b'4
            c''8
            d''16
            e''8.
            ~
            e''16
            f''8.
        }
        """
    )


def test_prettify_rewrite_meter_11():
    staff = abjad.Staff(
        r"\time 3/4 c'8 d'4 e'4 f'16 g'16 | "
        r"\time 4/4 a'8 b'4 c''8 d''16 e''4 f''8."
    )
    meters = [abjad.Meter((3, 4)), abjad.Meter((4, 4))]
    for meter, measure in zip(meters,
                              abjad.select(staff[:]).group_by_measure(),
                              ):
        abjad.Meter.rewrite_meter(measure, meter)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            c'8
            d'8
            ~
            d'8
            e'8
            ~
            e'8
            f'16
            g'16
            \time 4/4
            a'8
            b'8
            ~
            b'8
            c''8
            d''16
            e''8.
            ~
            e''16
            f''8.
        }
        """
    )
    for meter, measure in zip(meters,
                              abjad.select(staff[:]).group_by_measure(),
                              ):
        auxjad.mutate.prettify_rewrite_meter(measure, meter)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            c'8
            d'4
            e'4
            f'16
            g'16
            \time 4/4
            a'8
            b'4
            c''8
            d''16
            e''8.
            ~
            e''16
            f''8.
        }
        """
    )


def test_prettify_rewrite_meter_12():
    container1 = abjad.Staff(
        r"\time 4/4 c'8 d'8 ~ d'8 e'8 c'16 d'16 ~ d'16 e'16 r4 "
        r"c'8 d'8 ~ d'8 e'8 c'16 d'16 ~ d'16 e'16 r4"
    )
    container2 = abjad.mutate.copy(container1)
    meter = abjad.Meter((4, 4))
    abjad.mutate.prettify_rewrite_meter(container1[:], meter)
    for measure in abjad.select(container2[:]).group_by_measure():
        abjad.mutate.prettify_rewrite_meter(measure, meter)
    selections = [container1[:], container2[:]]
    assert auxjad.get.selections_are_identical(selections)


def test_prettify_rewrite_meter_13():
    staff = abjad.Staff(
        r"\times 2/3 {c'4 ~ c'8} \times 2/3 {d'8 r4} "
        r"\times 2/3 {r8 r8 r8} \times 2/3 {<e' g'>8 ~ <e' g'>4}"
    )
    meter = abjad.Meter((4, 4))
    abjad.Meter.rewrite_meter(staff[:], meter)
    abjad.mutate.prettify_rewrite_meter(staff[:], meter)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'4
            \times 2/3
            {
                d'8
                r4
            }
            r4
            <e' g'>4
        }
        """
    )


def test_prettify_rewrite_meter_14():
    staff = abjad.Staff(
        r"\times 2/3 {c'4 ~ c'8} \times 2/3 {d'8 r4} "
        r"\times 2/3 {r8 r8 r8} \times 2/3 {<e' g'>8 ~ <e' g'>4}"
    )
    meter = abjad.Meter((4, 4))
    abjad.Meter.rewrite_meter(staff[:], meter)
    abjad.mutate.prettify_rewrite_meter(
        staff[:],
        meter,
        extract_trivial_tuplets=False,
    )
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \times 2/3
            {
                c'4.
            }
            \times 2/3
            {
                d'8
                r4
            }
            \times 2/3
            {
                r4.
            }
            \times 2/3
            {
                <e' g'>4.
            }
        }
        """
    )


def test_prettify_rewrite_meter_15():
    staff = abjad.Staff(
        r"c'4 d'2 r4"
        r"e'4. f'2 g'8"
        r"a'4. b'4. c''4"
        r"d''16 e''8. f''4. g''4 a''8"
    )
    meter = abjad.Meter((4, 4))
    for measure in abjad.select(staff[:]).group_by_measure():
        abjad.Meter.rewrite_meter(measure, meter)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'4
            d'2
            r4
            e'4.
            f'8
            ~
            f'4.
            g'8
            a'4.
            b'4.
            c''4
            d''16
            e''8.
            f''4.
            g''8
            ~
            g''8
            a''8
        }
        """
    )
    abjad.mutate.prettify_rewrite_meter(staff[:], meter)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'4
            d'2
            r4
            e'4.
            f'8
            ~
            f'4.
            g'8
            a'4.
            b'8
            ~
            b'4
            c''4
            d''16
            e''8.
            f''4
            ~
            f''8
            g''4
            a''8
        }
        """
    )
    staff = abjad.Staff(
        r"c'4 d'2 r4"
        r"e'4. f'2 g'8"
        r"a'4. b'4. c''4"
        r"d''16 e''8. f''4. g''4 a''8"
    )
    meter = abjad.Meter((4, 4))
    for measure in abjad.select(staff[:]).group_by_measure():
        abjad.Meter.rewrite_meter(measure, meter)
    abjad.mutate.prettify_rewrite_meter(
        staff[:],
        meter,
        split_quadruple_meter=False,
    )
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'4
            d'2
            r4
            e'4.
            f'8
            ~
            f'4.
            g'8
            a'4.
            b'4.
            c''4
            d''16
            e''8.
            f''4.
            g''4
            a''8
        }
        """
    )


def test_prettify_rewrite_meter_16():
    staff = abjad.Staff(
        r"\time 3/4 c'16 d'8 e'16 f'16 g'16 a'8 b'8 c''16 d''16"
    )
    meter = abjad.Meter((3, 4))
    abjad.Meter.rewrite_meter(staff[:], meter)
    abjad.mutate.prettify_rewrite_meter(staff[:], meter)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            c'16
            d'8
            e'16
            f'16
            g'16
            a'8
            b'8
            c''16
            d''16
        }
        """
    )



import abjad

import auxjad


def test_example_of_usage_01():
    material = abjad.Staff(r"\time 12/8 c8 c c r c c r c r c c r",
                           lilypond_type="RhythmicStaff",
                           )
    phaser = auxjad.Phaser(material,
                           step_size=abjad.Duration((1, 8)),
                           )
    notes = phaser.output_all()
    phased_staff = abjad.Staff(notes,
                               lilypond_type="RhythmicStaff",
                               )
    repeater = auxjad.Repeater(material)
    notes = repeater(13)
    constant_staff = abjad.Staff(notes,
                                 lilypond_type="RhythmicStaff",
                                 )
    score = abjad.Score([constant_staff, phased_staff])
    measures = abjad.select(constant_staff[:]).group_by_measure()
    for measure in measures[:-1]:
        abjad.attach(abjad.BarLine(':..:'), measure[-1])
    abjad.attach(abjad.BarLine(':|.'), constant_staff[-1])
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new RhythmicStaff
            {
                \time 12/8
                c8
                c8
                c8
                r8
                c8
                c8
                r8
                c8
                r8
                c8
                c8
                r8
                \bar ":..:"
                c8
                c8
                c8
                r8
                c8
                c8
                r8
                c8
                r8
                c8
                c8
                r8
                \bar ":..:"
                c8
                c8
                c8
                r8
                c8
                c8
                r8
                c8
                r8
                c8
                c8
                r8
                \bar ":..:"
                c8
                c8
                c8
                r8
                c8
                c8
                r8
                c8
                r8
                c8
                c8
                r8
                \bar ":..:"
                c8
                c8
                c8
                r8
                c8
                c8
                r8
                c8
                r8
                c8
                c8
                r8
                \bar ":..:"
                c8
                c8
                c8
                r8
                c8
                c8
                r8
                c8
                r8
                c8
                c8
                r8
                \bar ":..:"
                c8
                c8
                c8
                r8
                c8
                c8
                r8
                c8
                r8
                c8
                c8
                r8
                \bar ":..:"
                c8
                c8
                c8
                r8
                c8
                c8
                r8
                c8
                r8
                c8
                c8
                r8
                \bar ":..:"
                c8
                c8
                c8
                r8
                c8
                c8
                r8
                c8
                r8
                c8
                c8
                r8
                \bar ":..:"
                c8
                c8
                c8
                r8
                c8
                c8
                r8
                c8
                r8
                c8
                c8
                r8
                \bar ":..:"
                c8
                c8
                c8
                r8
                c8
                c8
                r8
                c8
                r8
                c8
                c8
                r8
                \bar ":..:"
                c8
                c8
                c8
                r8
                c8
                c8
                r8
                c8
                r8
                c8
                c8
                r8
                \bar ":..:"
                c8
                c8
                c8
                r8
                c8
                c8
                r8
                c8
                r8
                c8
                c8
                r8
                \bar ":|."
            }
            \new RhythmicStaff
            {
                \time 12/8
                c8
                c8
                c8
                r8
                c8
                c8
                r8
                c8
                r8
                c8
                c8
                r8
                c8
                c8
                r8
                c8
                c8
                r8
                c8
                r8
                c8
                c8
                r8
                c8
                c8
                r8
                c8
                c8
                r8
                c8
                r8
                c8
                c8
                r8
                c8
                c8
                r8
                c8
                c8
                r8
                c8
                r8
                c8
                c8
                r8
                c8
                c8
                c8
                c8
                c8
                r8
                c8
                r8
                c8
                c8
                r8
                c8
                c8
                c8
                r8
                c8
                r8
                c8
                r8
                c8
                c8
                r8
                c8
                c8
                c8
                r8
                c8
                r8
                c8
                r8
                c8
                c8
                r8
                c8
                c8
                c8
                r8
                c8
                c8
                c8
                r8
                c8
                c8
                r8
                c8
                c8
                c8
                r8
                c8
                c8
                r8
                r8
                c8
                c8
                r8
                c8
                c8
                c8
                r8
                c8
                c8
                r8
                c8
                c8
                c8
                r8
                c8
                c8
                c8
                r8
                c8
                c8
                r8
                c8
                r8
                c8
                r8
                c8
                c8
                c8
                r8
                c8
                c8
                r8
                c8
                r8
                c8
                r8
                c8
                c8
                c8
                r8
                c8
                c8
                r8
                c8
                r8
                c8
                c8
                c8
                c8
                c8
                r8
                c8
                c8
                r8
                c8
                r8
                c8
                c8
                r8
            }
        >>
        """
    )



import abjad
import pytest

import auxjad


def test_ArtificialHarmonic_01():
    harm = auxjad.ArtificialHarmonic(r"<g c'>4")
    assert harm.style == "#'harmonic"
    assert abjad.lilypond(harm) == abjad.String.normalize(
        r"""
        <
            g
            \tweak style #'harmonic
            c'
        >4
        """
    )


def test_ArtificialHarmonic_02():
    harm1 = auxjad.ArtificialHarmonic(r"<g c'>4")
    harm2 = auxjad.ArtificialHarmonic(["g", "c'"], 1 / 4)
    harm3 = auxjad.ArtificialHarmonic([-5, 0], 0.25)
    harm4 = auxjad.ArtificialHarmonic([-5, 0], abjad.Duration(1, 4))
    harms = [harm1, harm2, harm3, harm4]
    for harm in harms:
        assert abjad.lilypond(harm) == abjad.String.normalize(
            r"""
            <
                g
                \tweak style #'harmonic
                c'
            >4
            """
        )


def test_ArtificialHarmonic_03():
    harm = auxjad.ArtificialHarmonic(r"<g c'>4",
                                     style="#'harmonic-mixed"
                                     )
    assert harm.style == "#'harmonic-mixed"
    assert abjad.lilypond(harm) == abjad.String.normalize(
        r"""
        <
            g
            \tweak style #'harmonic-mixed
            c'
        >4
        """
    )


def test_ArtificialHarmonic_04():
    harm = auxjad.ArtificialHarmonic(r"<g c'>4",
                                     is_parenthesized=True,
                                     )
    assert harm.is_parenthesized
    assert abjad.lilypond(harm) == abjad.String.normalize(
        r"""
        <
            \parenthesize
            \tweak ParenthesesItem.font-size -4
            g
            \tweak style #'harmonic
            c'
        >4
        """
    )


def test_ArtificialHarmonic_05():
    harm = auxjad.ArtificialHarmonic(r"<g c'>4",
                                     multiplier=(2, 3),
                                     )
    assert harm.multiplier == abjad.Multiplier(2, 3)
    assert abjad.lilypond(harm) == abjad.String.normalize(
        r"""
        <
            g
            \tweak style #'harmonic
            c'
        >4 * 2/3
        """
    )


def test_ArtificialHarmonic_06():
    harm = auxjad.ArtificialHarmonic(r"<g c'>4")
    assert harm.written_pitches == abjad.PitchSegment(r"g c'")
    assert harm.written_duration == 1 / 4
    assert harm.style == "#'harmonic"
    assert not harm.is_parenthesized
    harm.written_pitches = [-5, 2]
    harm.written_duration = abjad.Duration(1, 8)
    harm.style = "#'harmonic-mixed"
    harm.is_parenthesized = True
    assert harm.written_pitches == abjad.PitchSegment(r"g d'")
    assert harm.written_duration == 1 / 8
    assert harm.style == "#'harmonic-mixed"
    assert harm.is_parenthesized


def test_ArtificialHarmonic_07():
    with pytest.raises(ValueError):
        harm = auxjad.ArtificialHarmonic(r"<g c' d'>4")  # noqa: F841


def test_ArtificialHarmonic_08():
    harmonics = [auxjad.ArtificialHarmonic(r"<g b>4").sounding_pitch(),
                 auxjad.ArtificialHarmonic(r"<g c'>4").sounding_pitch(),
                 auxjad.ArtificialHarmonic(r"<g d'>4").sounding_pitch(),
                 auxjad.ArtificialHarmonic(r"<g e'>4").sounding_pitch(),
                 auxjad.ArtificialHarmonic(r"<g g'>4").sounding_pitch(),
                 ]
    notes = [abjad.Note(r"b''4"),
             abjad.Note(r"g''4"),
             abjad.Note(r"d''4"),
             abjad.Note(r"b''4"),
             abjad.Note(r"g'4"),
             ]
    for harmonic_pitch, note in zip(harmonics, notes):
        assert harmonic_pitch == note.written_pitch


def test_ArtificialHarmonic_09():
    harm = auxjad.ArtificialHarmonic(r"<g ef'>4")
    with pytest.raises(ValueError):
        harm.sounding_pitch()


def test_ArtificialHarmonic_10():
    harmonics = [auxjad.ArtificialHarmonic(r"<g b>4").sounding_note(),
                 auxjad.ArtificialHarmonic(r"<g c'>4").sounding_note(),
                 auxjad.ArtificialHarmonic(r"<g d'>4").sounding_note(),
                 auxjad.ArtificialHarmonic(r"<g e'>4").sounding_note(),
                 auxjad.ArtificialHarmonic(r"<g g'>4").sounding_note(),
                 ]
    notes = [abjad.Note(r"b''4"),
             abjad.Note(r"g''4"),
             abjad.Note(r"d''4"),
             abjad.Note(r"b''4"),
             abjad.Note(r"g'4"),
             ]
    for harmonic, note in zip(harmonics, notes):
        assert harmonic.written_pitch == note.written_pitch
        assert harmonic.written_duration == note.written_duration


def test_ArtificialHarmonic_11():
    harm = auxjad.ArtificialHarmonic(r"<g c'>4-.\pp")
    assert abjad.lilypond(harm.sounding_note()) == abjad.String.normalize(
        r"""
        g''4
        \pp
        - \staccato
        """
    )


def test_ArtificialHarmonic_12():
    harm = auxjad.ArtificialHarmonic(r"<g ef'>4")
    with pytest.raises(ValueError):
        harm.sounding_note()


def test_ArtificialHarmonic_13():
    harm = auxjad.ArtificialHarmonic(r"<g c'>4")
    assert harm.written_pitches == abjad.PitchSegment(r"g c'")
    harm.written_pitches = r"a d'"
    assert harm.written_pitches == abjad.PitchSegment(r"a d'")
    with pytest.raises(ValueError):
        harm.written_pitches = r"a d' e'"


def test_ArtificialHarmonic_14():
    harm1 = auxjad.ArtificialHarmonic(r"<a d'>1")
    harm2 = auxjad.ArtificialHarmonic(r"<a d'>1",
                                      markup='I.',
                                      )
    harm3 = auxjad.ArtificialHarmonic(r"<a d'>1",
                                      markup='I.',
                                      direction=abjad.Down)
    staff = abjad.Staff([harm1, harm2, harm3])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            <
                a
                \tweak style #'harmonic
                d'
            >1
            \once \override TextScript.parent-alignment-X = 0
            \once \override TextScript.self-alignment-X = 0
            <
                a
                \tweak style #'harmonic
                d'
            >1
            ^ \markup { I. }
            \once \override TextScript.parent-alignment-X = 0
            \once \override TextScript.self-alignment-X = 0
            <
                a
                \tweak style #'harmonic
                d'
            >1
            _ \markup { I. }
        }
        """
    )


def test_ArtificialHarmonic_15():
    harm = auxjad.ArtificialHarmonic(r"<a d'>1",
                                     markup='I.',
                                     )
    harm.markup = None
    assert abjad.lilypond(harm) == abjad.String.normalize(
        r"""
        <
            a
            \tweak style #'harmonic
            d'
        >1
        """
    )


def test_ArtificialHarmonic_16():
    harm = auxjad.ArtificialHarmonic(r"<a d'>1")
    abjad.attach(abjad.Markup('test'), harm)
    harm.markup = 'I.'
    with pytest.raises(Exception):
        harm.markup = None


def test_ArtificialHarmonic_17():
    harm = auxjad.ArtificialHarmonic(r"<a d'>1",
                                     markup='I.',
                                     direction=abjad.Down)
    assert harm.direction is abjad.Down
    assert abjad.lilypond(harm) == abjad.String.normalize(
        r"""
        \once \override TextScript.parent-alignment-X = 0
        \once \override TextScript.self-alignment-X = 0
        <
            a
            \tweak style #'harmonic
            d'
        >1
        _ \markup { I. }
        """
    )
    harm.direction = abjad.Up
    assert harm.direction is abjad.Up
    assert abjad.lilypond(harm) == abjad.String.normalize(
        r"""
        \once \override TextScript.parent-alignment-X = 0
        \once \override TextScript.self-alignment-X = 0
        <
            a
            \tweak style #'harmonic
            d'
        >1
        ^ \markup { I. }
        """
    )


def test_ArtificialHarmonic_18():
    harm1 = auxjad.ArtificialHarmonic(r"<bf' ef''>1")
    harm2 = auxjad.ArtificialHarmonic(r"<bf' ef''>1",
                                      markup='III.',
                                      )
    harm3 = auxjad.ArtificialHarmonic(r"<bf' ef''>1",
                                      markup='III.',
                                      centre_markup=True,
                                      )
    harm4 = auxjad.ArtificialHarmonic(r"<bf' ef''>1",
                                      markup='III.',
                                      centre_markup=False,
                                      )
    assert abjad.lilypond(harm1) == abjad.String.normalize(
        r"""
        <
            bf'
            \tweak style #'harmonic
            ef''
        >1
        """
    )
    assert abjad.lilypond(harm2) == abjad.String.normalize(
        r"""
        \once \override TextScript.parent-alignment-X = 0
        \once \override TextScript.self-alignment-X = 0
        <
            bf'
            \tweak style #'harmonic
            ef''
        >1
        ^ \markup { III. }
        """
    )
    assert abjad.lilypond(harm3) == abjad.String.normalize(
        r"""
        \once \override TextScript.parent-alignment-X = 0
        \once \override TextScript.self-alignment-X = 0
        <
            bf'
            \tweak style #'harmonic
            ef''
        >1
        ^ \markup { III. }
        """
    )
    assert abjad.lilypond(harm4) == abjad.String.normalize(
        r"""
        <
            bf'
            \tweak style #'harmonic
            ef''
        >1
        ^ \markup { III. }
        """
    )



import random

import abjad
import pytest

import auxjad


def test_Shuffler_01():
    random.seed(87234)
    container = abjad.Container(r"c'4 d'4 e'4 f'4")
    shuffler = auxjad.Shuffler(container)
    assert abjad.lilypond(shuffler) == abjad.String.normalize(
        r"""
        {
            c'4
            d'4
            e'4
            f'4
        }
        """
    )
    notes = shuffler()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            d'4
            c'4
            f'4
            e'4
        }
        """
    )
    notes = shuffler.shuffle()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'4
            e'4
            d'4
            f'4
        }
        """
    )
    notes = shuffler.current_window
    with pytest.raises(AttributeError):
        shuffler.current_window = abjad.Container(r"c''2 e''2")
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'4
            e'4
            d'4
            f'4
        }
        """
    )


def test_Shuffler_02():
    container = abjad.Container(r"\time 3/4 c'4 d'4 e'4 \time 2/4 f'4 g'4")
    shuffler = auxjad.Shuffler(container,
                               pitch_only=False,
                               preserve_rest_position=True,
                               disable_rewrite_meter=False,
                               omit_time_signatures=True,
                               boundary_depth=0,
                               maximum_dot_count=1,
                               rewrite_tuplets=False,
                               process_on_first_call=True,
                               swap_limit=3,
                               )
    assert not shuffler.pitch_only
    assert shuffler.preserve_rest_position
    assert not shuffler.disable_rewrite_meter
    assert shuffler.omit_time_signatures
    assert shuffler.boundary_depth == 0
    assert shuffler.maximum_dot_count == 1
    assert not shuffler.rewrite_tuplets
    assert shuffler.process_on_first_call
    assert shuffler.swap_limit == 3
    shuffler.pitch_only = True
    shuffler.preserve_rest_position = False
    shuffler.disable_rewrite_meter = True
    shuffler.omit_time_signatures = False
    shuffler.boundary_depth = 1
    shuffler.maximum_dot_count = 2
    shuffler.rewrite_tuplets = True
    shuffler.process_on_first_call = False
    shuffler.swap_limit = None
    assert shuffler.pitch_only
    assert not shuffler.preserve_rest_position
    assert shuffler.disable_rewrite_meter
    assert not shuffler.omit_time_signatures
    assert shuffler.boundary_depth == 1
    assert shuffler.maximum_dot_count == 2
    assert shuffler.rewrite_tuplets
    assert not shuffler.process_on_first_call
    assert shuffler.swap_limit is None


def test_Shuffler_03():
    random.seed(18331)
    container = abjad.Container(r"c'8. d'4 r8 r8. e'16 f'8.")
    shuffler = auxjad.Shuffler(container)
    notes = shuffler()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            r4
            r16
            e'16
            f'8
            ~
            f'16
            d'8.
            ~
            d'16
            c'8.
        }
        """
    )
    container = abjad.Container(r"c'8. d'4 r8 r8. e'16 f'8.")
    shuffler = auxjad.Shuffler(container, pitch_only=True)
    notes = shuffler()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            f'8.
            r4
            d'8
            ~
            d'8.
            c'16
            e'8.
        }
        """
    )


def test_Shuffler_04():
    container = abjad.Container(
        r"\time 3/4 c'16 d'8. ~ d'4 e'4 r4 f'4 ~ f'8.. g'32"
    )
    shuffler = auxjad.Shuffler(container)
    notes = shuffler.rotate()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            d'4..
            e'16
            ~
            e'8.
            r16
            r8.
            f'16
            ~
            f'4
            ~
            f'8
            ~
            f'32
            g'32
            c'16
        }
        """
    )
    notes = shuffler.rotate(n_rotations=2, anticlockwise=True)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            g'32
            c'16
            d'32
            ~
            d'8
            ~
            d'4
            ~
            d'32
            e'8..
            ~
            e'32
            r8..
            r32
            f'4...
        }
        """
    )
    container = abjad.Container(
        r"\time 3/4 c'16 d'8. ~ d'4 e'4 r4 f'4 ~ f'8.. g'32"
    )
    shuffler = auxjad.Shuffler(container, pitch_only=True)
    notes = shuffler.rotate()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            d'16
            e'8.
            ~
            e'4
            r4
            f'4
            g'4
            ~
            g'8..
            c'32
        }
        """
    )
    notes = shuffler.rotate(n_rotations=2, anticlockwise=True)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            g'16
            c'8.
            ~
            c'4
            d'4
            e'4
            r4
            r8..
            f'32
        }
        """
    )


def test_Shuffler_05():
    random.seed(18332)
    container = abjad.Container(r"c'8. d'4 r8 r8. e'16 f'8.")
    shuffler = auxjad.Shuffler(container, preserve_rest_position=True)
    notes = shuffler()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            d'4
            e'16
            r8.
            r8
            f'8
            ~
            f'16
            c'8.
        }
        """
    )
    container = abjad.Container(r"c'8. d'4 r8 r8. e'16 f'8.")
    shuffler = auxjad.Shuffler(container,
                               pitch_only=True,
                               preserve_rest_position=True,
                               )
    notes = shuffler()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            d'8.
            f'4
            r8
            r8.
            c'16
            e'8.
        }
        """
    )


def test_Shuffler_06():
    container = abjad.Container(r"c'8. d'4 r8 r8. e'16 f'8.")
    shuffler = auxjad.Shuffler(container, preserve_rest_position=True)
    notes = shuffler.rotate()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            d'4
            e'16
            r8.
            r8
            f'8
            ~
            f'16
            c'8.
        }
        """
    )
    container = abjad.Container(r"c'8. d'4 r8 r8. e'16 f'8.")
    shuffler = auxjad.Shuffler(container,
                               pitch_only=True,
                               preserve_rest_position=True,
                               )
    notes = shuffler.rotate()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            d'8.
            e'4
            r8
            r8.
            f'16
            c'8.
        }
        """
    )


def test_Shuffler_07():
    random.seed(98231)
    container = abjad.Container(r"c'4 d'8 e'4. f'8. g'16")
    shuffler = auxjad.Shuffler(container)
    notes = shuffler.shuffle_n(2)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            d'8
            g'16
            c'16
            ~
            c'8.
            f'16
            ~
            f'8
            e'4.
            g'16
            f'8.
            e'4
            ~
            e'8
            c'4
            d'8
        }
        """
    )
    container = abjad.Container(r"c'4 d'8 e'4. f'8. g'16")
    shuffler = auxjad.Shuffler(container, pitch_only=True)
    notes = shuffler.shuffle_n(2)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            e'4
            d'8
            g'4.
            c'8.
            f'16
            f'4
            c'8
            d'4.
            e'8.
            g'16
        }
        """
    )


def test_Shuffler_08():
    container = abjad.Container(r"c'4 d'8 e'4. f'8. g'16")
    shuffler = auxjad.Shuffler(container)
    notes = shuffler.rotate_n(2)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            d'8
            e'4.
            f'8.
            g'16
            c'4
            e'4.
            f'8
            ~
            f'16
            g'16
            c'4
            d'8
        }
        """
    )
    container = abjad.Container(r"c'4 d'8 e'4. f'8. g'16")
    shuffler = auxjad.Shuffler(container, pitch_only=True)
    notes = shuffler.rotate_n(2)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            d'4
            e'8
            f'4.
            g'8.
            c'16
            e'4
            f'8
            g'4.
            c'8.
            d'16
        }
        """
    )


def test_Shuffler_09():
    random.seed(17231)
    container = abjad.Container(r"\time 5/4 r4 \times 2/3 {c'4 d'2} e'4. f'8")
    shuffler = auxjad.Shuffler(container, pitch_only=True)
    notes = shuffler()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 5/4
            d'4
            \times 2/3
            {
                f'4
                c'2
            }
            e'4.
            r8
        }
        """
    )
    shuffler.pitch_only = False
    with pytest.raises(ValueError):
        notes = shuffler()  # noqa: F841
        notes = shuffler.rotate()  # noqa: F841


def test_Shuffler_10():
    random.seed(98103)
    container = abjad.Container(
        r"\time 3/4 c'8. d'4 r8 r8. \time 2/4 e'16 f'4.."
    )
    shuffler = auxjad.Shuffler(container)
    notes = shuffler.shuffle_n(2)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            e'16
            d'8.
            ~
            d'16
            f'4..
            \time 2/4
            c'8.
            r16
            r4
            \time 3/4
            c'8.
            f'16
            ~
            f'4.
            r8
            \time 2/4
            r8.
            d'16
            ~
            d'8.
            e'16
        }
        """
    )


def test_Shuffler_11():
    random.seed(87233)
    container = abjad.Container(r"<c' e' g'>4--\p d'8-. e'8-. f'4-^\f r4")
    shuffler = auxjad.Shuffler(container)
    notes = shuffler.shuffle_n(3)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            e'8
            \p
            - \staccato
            d'8
            - \staccato
            f'4
            \f
            - \marcato
            <c' e' g'>4
            \p
            - \tenuto
            r4
            r4
            d'8
            - \staccato
            f'8
            \f
            - \marcato
            ~
            f'8
            <c' e' g'>4
            \p
            - \tenuto
            e'8
            - \staccato
            f'4
            \f
            - \marcato
            e'8
            \p
            - \staccato
            <c' e' g'>8
            - \tenuto
            ~
            <c' e' g'>8
            d'8
            - \staccato
            r4
        }
        """
    )


def test_Shuffler_12():
    random.seed(18892)
    container = abjad.Container(r"\time 3/4 c'16 d'4.. e'4 | r4 f'2")
    shuffler = auxjad.Shuffler(container,
                               omit_time_signatures=True,
                               )
    notes = shuffler()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            d'4..
            e'16
            ~
            e'8.
            f'16
            ~
            f'4..
            r16
            r8.
            c'16
        }
        """
    )
    assert shuffler.omit_time_signatures
    shuffler.omit_time_signatures = False
    assert not shuffler.omit_time_signatures


def test_Shuffler_13():
    container = abjad.Container(r"c'4 d'4 e'4 f'4")
    shuffler = auxjad.Shuffler(container)
    assert abjad.lilypond(shuffler.contents) == abjad.String.normalize(
        r"""
        {
            c'4
            d'4
            e'4
            f'4
        }
        """
    )
    shuffler()
    assert abjad.lilypond(shuffler.contents) == abjad.String.normalize(
        r"""
        {
            c'4
            d'4
            e'4
            f'4
        }
        """
    )
    shuffler.contents = abjad.Container(r"cs2 ds2")
    assert abjad.lilypond(shuffler.contents) == abjad.String.normalize(
        r"""
        {
            cs2
            ds2
        }
        """
    )


def test_Shuffler_14():
    container = abjad.Container(r"c'2 d'2")
    shuffler = auxjad.Shuffler(container)
    assert len(shuffler) == 2
    container = abjad.Container(r"c'4 d'4 e'4 f'4 ~ | f'2 g'2")
    shuffler = auxjad.Shuffler(container)
    assert len(shuffler) == 5
    container = abjad.Container(r"c'8. d'4 r8 r8. e'16 f'8.")
    shuffler = auxjad.Shuffler(container)
    assert len(shuffler) == 5


def test_Shuffler_15():
    random.seed(98604)
    container = abjad.Container(r"c'4. d'8 e'2")
    shuffler = auxjad.Shuffler(container)
    notes = shuffler()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            e'2
            c'4.
            d'8
        }
        """
    )
    random.seed(98604)
    shuffler = auxjad.Shuffler(container,
                               boundary_depth=1,
                               )
    notes = shuffler()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            e'2
            c'4
            ~
            c'8
            d'8
        }
        """
    )


def test_Shuffler_16():
    random.seed(19867)
    container = abjad.Container(r"c'4 d'8. e'16 f'2")
    shuffler = auxjad.Shuffler(container)
    notes = shuffler()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            e'16
            f'8.
            ~
            f'4
            ~
            f'16
            c'8.
            ~
            c'16
            d'8.
        }
        """
    )
    random.seed(19867)
    shuffler = auxjad.Shuffler(container,
                               disable_rewrite_meter=True,
                               )
    notes = shuffler()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            e'16
            f'2
            c'4
            d'8.
        }
        """
    )


def test_Shuffler_17():
    random.seed(22047)
    container = abjad.Container(r"c'4 d'4 e'4 f'4")
    shuffler = auxjad.Shuffler(container,
                               process_on_first_call=False,
                               )
    notes = shuffler()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'4
            d'4
            e'4
            f'4
        }
        """
    )


def test_Shuffler_18():
    random.seed(10932)
    container = abjad.Container(r"\time 3/4 c'4 d'4 e'4")
    shuffler = auxjad.Shuffler(container)
    staff = abjad.Staff()
    for window in shuffler:
        staff.append(window)
        if abjad.get.duration(staff) == abjad.Duration((9, 4)):
            break
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            e'4
            c'4
            d'4
            \time 3/4
            d'4
            c'4
            e'4
            \time 3/4
            c'4
            e'4
            d'4
        }
        """
    )
    auxjad.mutate.remove_repeated_time_signatures(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            e'4
            c'4
            d'4
            d'4
            c'4
            e'4
            c'4
            e'4
            d'4
        }
        """
    )


def test_Shuffler_19():
    container = abjad.Container(r"c'4 d'4 e'4 f'4")
    shuffler = auxjad.Shuffler(container)
    assert isinstance(shuffler(), abjad.Selection)
    tuplet = abjad.Tuplet('3:2', r"c'2 d'2 e'2")
    shuffler = auxjad.Shuffler(tuplet, pitch_only=True)
    assert isinstance(shuffler(), abjad.Selection)
    voice = abjad.Voice(r"c'4 d'4 e'4 f'4")
    shuffler = auxjad.Shuffler(voice)
    assert isinstance(shuffler(), abjad.Selection)
    staff = abjad.Staff(r"c'4 d'4 e'4 f'4")
    shuffler = auxjad.Shuffler(staff)
    assert isinstance(shuffler(), abjad.Selection)
    score = abjad.Score([abjad.Staff(r"c'4 d'4 e'4 f'4")])
    shuffler = auxjad.Shuffler(score)
    assert isinstance(shuffler(), abjad.Selection)
    voice = abjad.Voice(r"c'4 d'4 e'4 f'4")
    staff = abjad.Staff([voice])
    shuffler = auxjad.Shuffler(staff)
    assert isinstance(shuffler(), abjad.Selection)
    staff = abjad.Staff(r"c'4 d'4 e'4 f'4")
    score = abjad.Score([staff])
    shuffler = auxjad.Shuffler(score)
    assert isinstance(shuffler(), abjad.Selection)

    voice1 = abjad.Voice(r"c'4 d'4 e'4 f'4")
    voice2 = abjad.Voice(r"g2 f2")
    staff = abjad.Staff([voice1, voice2], simultaneous=True)
    with pytest.raises(ValueError):
        shuffler = auxjad.Shuffler(staff)  # noqa: F841

    staff1 = abjad.Staff(r"c'4 d'4 e'4 f'4")
    staff2 = abjad.Staff(r"g2 f2")
    score = abjad.Score([staff1, staff2])
    with pytest.raises(ValueError):
        shuffler = auxjad.Shuffler(score)  # noqa: F841


def test_Shuffler_20():
    random.seed(76123)
    container = abjad.Container(r"\times 2/3 {\time 3/4 r8 d'8 r8} c'4 r4")
    shuffler = auxjad.Shuffler(container, pitch_only=True)
    notes = shuffler.shuffle_n(5)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \times 2/3
            {
                \time 3/4
                r8
                d'8
                r8
            }
            r4
            c'4
            \times 2/3
            {
                d'8
                c'8
                r8
            }
            r4
            r4
            r4
            d'4
            c'4
            \times 2/3
            {
                r8
                d'8
                c'8
            }
            r4
            r4
            \times 2/3
            {
                c'8
                r8
                r8
            }
            d'4
            r4
        }
        """
    )


def test_Shuffler_21():
    random.seed(98141)
    container = abjad.Container(r"c'4.. d'16 e'4. f'8")
    shuffler = auxjad.Shuffler(container, pitch_only=True)
    notes = shuffler.shuffle_n(2)
    staff = abjad.Staff(notes)
    shuffler.pitch_only = False
    notes = shuffler.shuffle_n(2)
    staff.append(notes)
    auxjad.mutate.remove_repeated_time_signatures(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            d'4..
            c'16
            f'4.
            e'8
            d'4..
            f'16
            c'4.
            e'8
            f'16
            d'4..
            e'8
            c'4.
            c'4.
            d'8
            ~
            d'4
            ~
            d'16
            e'8
            f'16
        }
        """
    )


def test_Shuffler_22():
    random.seed(91288)
    container = abjad.Container(r"c'8 d'8 e'8 f'8 g'8 a'8 b'8 c''8")
    shuffler = auxjad.Shuffler(container,
                               swap_limit=1,
                               )
    notes = shuffler.shuffle_n(3)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'8
            d'8
            e'8
            g'8
            f'8
            a'8
            b'8
            c''8
            c'8
            c''8
            e'8
            g'8
            f'8
            a'8
            b'8
            d'8
            c'8
            c''8
            e'8
            g'8
            b'8
            a'8
            f'8
            d'8
        }
        """
    )



import abjad

import auxjad


def test_rests_to_multimeasure_rest_01():
    staff = abjad.Staff(r"r1")
    auxjad.mutate.rests_to_multimeasure_rest(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            R1
        }
        """
    )


def test_rests_to_multimeasure_rest_02():
    staff = abjad.Staff(r"r2 r8.. r32 r16 r8 r16")
    auxjad.mutate.rests_to_multimeasure_rest(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            R1
        }
        """
    )


def test_rests_to_multimeasure_rest_03():
    staff = abjad.Staff(
        r"\time 3/4 r2. | \time 6/8 r2. | \time 5/4 c'1 ~ c'4 | r1 r4"
    )
    auxjad.mutate.rests_to_multimeasure_rest(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            R1 * 3/4
            \time 6/8
            R1 * 3/4
            \time 5/4
            c'1
            ~
            c'4
            R1 * 5/4
        }
        """
    )


def test_rests_to_multimeasure_rest_04():
    staff = abjad.Staff(r"\times 2/3 {r2 r2 r2}")
    auxjad.mutate.rests_to_multimeasure_rest(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            R1
        }
        """
    )
    staff = abjad.Staff(
        r"r2 \times 2/3 {r2 r4} \times 4/5 {r2. \times 2/3 {r2 r4}}"
    )
    auxjad.mutate.rests_to_multimeasure_rest(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            R1
            R1
        }
        """
    )


def test_rests_to_multimeasure_rest_05():
    staff = abjad.Staff(
        r"r2 \times 2/3 {r2 r4} \times 4/5 {r2. \times 2/3 {r2 r4}}"
    )
    auxjad.mutate.rests_to_multimeasure_rest(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            R1
            R1
        }
        """
    )


def test_rests_to_multimeasure_rest_06():
    staff = abjad.Staff(r"r2 r8.. r32 r16 r8 r16")
    abjad.mutate.rests_to_multimeasure_rest(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            R1
        }
        """
    )


def test_rests_to_multimeasure_rest_07():
    staff = abjad.Staff(
        r"\clef bass r4 r4 \times 2/3 {r4 r8} r4 "
        r"\time 3/4 \clef treble r2. "
        r"\time 5/4 r2 \clef bass r2."
    )
    abjad.mutate.rests_to_multimeasure_rest(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \clef "bass"
            R1
            \time 3/4
            \clef "treble"
            R1 * 3/4
            \time 5/4
            \clef "bass"
            R1 * 5/4
        }
        """
    )
    staff = abjad.Staff(
        r"\clef bass r4 r4 \times 2/3 {r4 r8} r4 "
        r"\time 3/4 \clef treble r2. "
        r"\time 5/4 r2 \clef bass r2."
    )
    abjad.mutate.rests_to_multimeasure_rest(staff[:], ignore_clefs=True)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            R1
            \time 3/4
            R1 * 3/4
            \time 5/4
            R1 * 5/4
        }
        """
    )


def test_rests_to_multimeasure_rest_08():
    staff = abjad.Staff(r"c'1\p\< r2\! r2 d'1\f\> r2 r2\ppp")
    abjad.mutate.rests_to_multimeasure_rest(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            \p
            \<
            R1
            \!
            d'1
            \f
            \>
            R1
            \ppp
        }
        """
    )
    staff = abjad.Staff(r"c'1\p\< r2\! r2 d'1\f\> r2 r2\ppp")
    abjad.mutate.rests_to_multimeasure_rest(staff[:], ignore_dynamics=True)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            \p
            \<
            R1
            d'1
            \f
            \>
            R1
        }
        """
    )



import abjad
import pytest

import auxjad


def test_virtual_fundamental_01():
    pitches = abjad.PitchSegment(r"c'' g''")
    fundamental = auxjad.get.virtual_fundamental(pitches)
    assert fundamental == abjad.NamedPitch(r"c'")


def test_virtual_fundamental_02():
    pitches = abjad.PitchSegment(r"c'' e'' g''")
    fundamental = auxjad.get.virtual_fundamental(pitches)
    assert fundamental == abjad.NamedPitch(r"c")


def test_virtual_fundamental_03():
    pitches = abjad.PitchSegment(r"c'' f'' g''")
    fundamental = auxjad.get.virtual_fundamental(pitches)
    assert fundamental == abjad.NamedPitch(r"f,")


def test_virtual_fundamental_04():
    pitches = abjad.PitchSegment(r"c'' d'' ef'' fs''")
    fundamental = auxjad.get.virtual_fundamental(pitches)
    assert fundamental == abjad.NamedPitch(r"bf,,")


def test_virtual_fundamental_05():
    pitches = abjad.PitchSegment(r"c'' cs'' d'' ef'' e'' fs''")
    fundamental = auxjad.get.virtual_fundamental(pitches)
    assert fundamental == abjad.NamedPitch(r"d,,")


def test_virtual_fundamental_06():
    chord = abjad.Chord(r"<c'' cs'' d'' ef'' e'' fs''>4")
    fundamental = auxjad.get.virtual_fundamental(chord)
    assert fundamental == abjad.NamedPitch(r"d,,")


def test_virtual_fundamental_07():
    staff = abjad.Staff(r"r4 <c'' cs'' d'' ef'' e'' fs''>4 r4")
    fundamental = auxjad.get.virtual_fundamental(staff[1])
    assert fundamental == abjad.NamedPitch(r"d,,")


def test_virtual_fundamental_08():
    pitches = abjad.PitchSegment(r"c'' cs'' d'' ef'' e'' fs''")
    fundamental = auxjad.get.virtual_fundamental(
        pitches,
        min_fundamental=abjad.NamedPitch(r"c,,,"),
    )
    assert fundamental == abjad.NamedPitch(r"d,,")


def test_virtual_fundamental_09():
    pitches = abjad.PitchSegment(r"c'' cs'' d'' ef'' e'' fs''")
    fundamental = auxjad.get.virtual_fundamental(
        pitches,
        min_fundamental=abjad.NumberedPitch(-48),
    )
    assert fundamental == abjad.NamedPitch(r"d,,")


def test_virtual_fundamental_10():
    pitches = abjad.PitchSegment(r"c'' cs'' d'' ef'' e'' fs''")
    with pytest.raises(ValueError):
        auxjad.get.virtual_fundamental(
            pitches,
            min_fundamental=abjad.NamedPitch(r"c'"),
        )


def test_virtual_fundamental_11():
    pitches = abjad.PitchSegment(r"c'' g''")
    fundamental = abjad.get.virtual_fundamental(pitches)
    assert fundamental == abjad.NamedPitch(r"c'")



import abjad

import auxjad


def test_staff_splitter_01():
    staff = abjad.Staff(r"a4 b4 c'4 d'4")
    staves = auxjad.staff_splitter(staff)
    score = abjad.Score(staves)
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                \clef "treble"
                r2
                c'4
                d'4
            }
            \new Staff
            {
                \clef "bass"
                a4
                b4
                r2
            }
        >>
        """
    )


def test_staff_splitter_02():
    staff = abjad.Staff(r"<g b>4 <a c'>4 <b d' f'>4 <a f c' e' g'>4")
    staves = auxjad.staff_splitter(staff)
    score = abjad.Score(staves)
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                \clef "treble"
                r4
                c'4
                <d' f'>4
                <c' e' g'>4
            }
            \new Staff
            {
                \clef "bass"
                <g b>4
                a4
                b4
                <f a>4
            }
        >>
        """
    )


def test_staff_splitter_03():
    staff = abjad.Staff(
        r"\time 2/4 c'2 \times 2/3 {<g b d'>2 <e' f'>4}"
        r"\times 2/3 {a2 <g b>4}"
    )
    staves = auxjad.staff_splitter(staff)
    score = abjad.Score(staves)
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                \time 2/4
                \clef "treble"
                c'2
                \times 2/3
                {
                    d'2
                    <e' f'>4
                }
                R1 * 1/2
            }
            \new Staff
            {
                \time 2/4
                \clef "bass"
                R1 * 1/2
                \times 2/3
                {
                    <g b>2
                    r4
                }
                \times 2/3
                {
                    a2
                    <g b>4
                }
            }
        >>
        """
    )


def test_staff_splitter_04():
    staff = abjad.Staff(r"c'4 d'4 e'4 <d' f' a'>4")
    staves = auxjad.staff_splitter(staff, threshold="e'")
    score = abjad.Score(staves)
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                \clef "treble"
                r2
                e'4
                <f' a'>4
            }
            \new Staff
            {
                \clef "bass"
                c'4
                d'4
                r4
                d'4
            }
        >>
        """
    )
    staves = auxjad.staff_splitter(staff, threshold='E4')
    score = abjad.Score(staves)
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                \clef "treble"
                r2
                e'4
                <f' a'>4
            }
            \new Staff
            {
                \clef "bass"
                c'4
                d'4
                r4
                d'4
            }
        >>
        """
    )
    staves = auxjad.staff_splitter(staff, threshold=abjad.NamedPitch("e'"))
    score = abjad.Score(staves)
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                \clef "treble"
                r2
                e'4
                <f' a'>4
            }
            \new Staff
            {
                \clef "bass"
                c'4
                d'4
                r4
                d'4
            }
        >>
        """
    )
    staves = auxjad.staff_splitter(staff, threshold=abjad.NumberedPitch(4))
    score = abjad.Score(staves)
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                \clef "treble"
                r2
                e'4
                <f' a'>4
            }
            \new Staff
            {
                \clef "bass"
                c'4
                d'4
                r4
                d'4
            }
        >>
        """
    )
    staves = auxjad.staff_splitter(staff, threshold=4)
    score = abjad.Score(staves)
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                \clef "treble"
                r2
                e'4
                <f' a'>4
            }
            \new Staff
            {
                \clef "bass"
                c'4
                d'4
                r4
                d'4
            }
        >>
        """
    )


def test_staff_splitter_05():
    staff = abjad.Staff(r"c'4 d'4 e'4 <d' f' a'>4")
    staves = auxjad.staff_splitter(staff,
                                   threshold="e'",
                                   lower_clef='treble',
                                   )
    score = abjad.Score(staves)
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                \clef "treble"
                r2
                e'4
                <f' a'>4
            }
            \new Staff
            {
                \clef "treble"
                c'4
                d'4
                r4
                d'4
            }
        >>
        """
    )


def test_staff_splitter_06():
    staff = abjad.Staff(r"e4 f4 g4 <f a c'>4")
    staves = auxjad.staff_splitter(staff,
                                   threshold='g',
                                   upper_clef='bass',
                                   )
    score = abjad.Score(staves)
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                \clef "bass"
                r2
                g4
                <a c'>4
            }
            \new Staff
            {
                \clef "bass"
                e4
                f4
                r4
                f4
            }
        >>
        """
    )


def test_staff_splitter_07():
    staff = abjad.Staff(r"c'4 d'4 e'4 <d' f' a'>4")
    staves = auxjad.staff_splitter(staff,
                                   threshold="e'",
                                   add_clefs=False,
                                   )
    score = abjad.Score(staves)
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                r2
                e'4
                <f' a'>4
            }
            \new Staff
            {
                c'4
                d'4
                r4
                d'4
            }
        >>
        """
    )


def test_staff_splitter_08():
    staff = abjad.Staff(
        r"\time 2/4 c'2 \times 2/3 {<g b d'>2 <e' f'>4}"
        r"\times 2/3 {a2 <g b>4}"
    )
    staves = auxjad.staff_splitter(staff,
                                   use_multimeasure_rests=False,
                                   )
    score = abjad.Score(staves)
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                \time 2/4
                \clef "treble"
                c'2
                \times 2/3
                {
                    d'2
                    <e' f'>4
                }
                r2
            }
            \new Staff
            {
                \time 2/4
                \clef "bass"
                r2
                \times 2/3
                {
                    <g b>2
                    r4
                }
                \times 2/3
                {
                    a2
                    <g b>4
                }
            }
        >>
        """
    )


def test_staff_splitter_09():
    staff = abjad.Staff(
        r"c'2\p <b d'>2\ff \times 2/3 {<g b d'>2\f <e' f'>1\mf}"
        r"\times 2/3 {a2\pp <g b>1\mp}"
    )
    staves = auxjad.staff_splitter(staff)
    score = abjad.Score(staves)
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                \clef "treble"
                c'2
                \p
                d'2
                \ff
                \times 2/3
                {
                    d'2
                    \f
                    <e' f'>1
                    \mf
                }
                R1
            }
            \new Staff
            {
                \clef "bass"
                r2
                b2
                \ff
                \times 2/3
                {
                    <g b>2
                    \f
                    r1
                }
                \times 2/3
                {
                    a2
                    \pp
                    <g b>1
                    \mp
                }
            }
        >>
        """
    )
    staff = abjad.Staff(
        r"c'2\p <b d'>2\ff \times 2/3 {<g b d'>2\f <e' f'>1\mf}"
        r"\times 2/3 {a2\pp <g b>1\mp}"
    )
    staves = auxjad.staff_splitter(staff,
                                   reposition_dynamics=False,
                                   )
    score = abjad.Score(staves)
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                \clef "treble"
                c'2
                \p
                d'2
                \ff
                \times 2/3
                {
                    d'2
                    \f
                    <e' f'>1
                    \mf
                }
                R1
                \mp
            }
            \new Staff
            {
                \clef "bass"
                r2
                \p
                b2
                \ff
                \times 2/3
                {
                    <g b>2
                    \f
                    r1
                    \mf
                }
                \times 2/3
                {
                    a2
                    \pp
                    <g b>1
                    \mp
                }
            }
        >>
        """
    )


def test_staff_splitter_10():
    staff = abjad.Staff(
        r"c'2\p <b d'>2\ff \times 2/3 {<g b d'>2\f <e' f'>1\mf}"
        r"\times 2/3 {a2\pp <g b>1\mp}"
    )
    staves = auxjad.staff_splitter(staff)
    score = abjad.Score(staves)
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                \clef "treble"
                c'2
                \p
                d'2
                \ff
                \times 2/3
                {
                    d'2
                    \f
                    <e' f'>1
                    \mf
                }
                R1
            }
            \new Staff
            {
                \clef "bass"
                r2
                b2
                \ff
                \times 2/3
                {
                    <g b>2
                    \f
                    r1
                }
                \times 2/3
                {
                    a2
                    \pp
                    <g b>1
                    \mp
                }
            }
        >>
        """
    )
    staff = abjad.Staff(
        r"c'2\p <b d'>2\ff \times 2/3 {<g b d'>2\f <e' f'>1\mf}"
        r"\times 2/3 {a2\pp <g b>1\mp}"
    )
    staves = auxjad.staff_splitter(staff,
                                   dynamics_only_on_upper_staff=True,
                                   )
    score = abjad.Score(staves)
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                \clef "treble"
                c'2
                \p
                d'2
                \ff
                \times 2/3
                {
                    d'2
                    \f
                    <e' f'>1
                    \mf
                }
                R1
            }
            \new Staff
            {
                \clef "bass"
                r2
                b2
                \times 2/3
                {
                    <g b>2
                    r1
                }
                \times 2/3
                {
                    a2
                    <g b>1
                }
            }
        >>
        """
    )
    staff = abjad.Staff(
        r"c'2\p <b d'>2\ff \times 2/3 {<g b d'>2\f <e' f'>1\mf}"
        r"\times 2/3 {a2\pp <g b>1\mp}"
    )
    staves = auxjad.staff_splitter(staff,
                                   dynamics_only_on_upper_staff=True,
                                   reposition_dynamics=False,
                                   )
    score = abjad.Score(staves)
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                \clef "treble"
                c'2
                \p
                d'2
                \ff
                \times 2/3
                {
                    d'2
                    \f
                    <e' f'>1
                    \mf
                }
                R1
                \mp
            }
            \new Staff
            {
                \clef "bass"
                r2
                b2
                \times 2/3
                {
                    <g b>2
                    r1
                }
                \times 2/3
                {
                    a2
                    <g b>1
                }
            }
        >>
        """
    )
    staff = abjad.Staff(
        r"c'2\p <b d'>2\ff \times 2/3 {<g b d'>2\f <e' f'>1\mf}"
        r"\times 2/3 {a2\pp <g b>1\mp}"
    )
    staves = auxjad.staff_splitter(staff,
                                   dynamics_only_on_upper_staff=True,
                                   reposition_dynamics=False,
                                   rewrite_meter=False,
                                   )
    score = abjad.Score(staves)
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                \clef "treble"
                c'2
                \p
                d'2
                \ff
                \times 2/3
                {
                    d'2
                    \f
                    <e' f'>1
                    \mf
                }
                R1
                \mp
            }
            \new Staff
            {
                \clef "bass"
                r2
                b2
                \times 2/3
                {
                    <g b>2
                    r1
                }
                \times 2/3
                {
                    a2
                    <g b>1
                }
            }
        >>
        """
    )


def test_staff_splitter_11():
    staff = abjad.Staff(r"a4 b4 c'4 d'4")
    staves = auxjad.staff_splitter(staff,
                                   rewrite_meter=False,
                                   )
    score = abjad.Score(staves)
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                \clef "treble"
                r4
                r4
                c'4
                d'4
            }
            \new Staff
            {
                \clef "bass"
                a4
                b4
                r4
                r4
            }
        >>
        """
    )


def test_staff_splitter_12():
    staff = abjad.Staff(
        r"\time 2/4 a8( b c' d') \times 2/3 {<g b d'>2 <e' f'>4}"
        r"\time 3/4 <d a c' g'>4--  r8 <f a bf>4."
    )
    staves = abjad.staff_splitter(staff)
    score = abjad.Score(staves)
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                \time 2/4
                \clef "treble"
                r4
                c'8
                (
                d'8
                )
                \times 2/3
                {
                    d'2
                    <e' f'>4
                }
                \time 3/4
                <c' g'>4
                - \tenuto
                r2
            }
            \new Staff
            {
                \time 2/4
                \clef "bass"
                a8
                (
                b8
                )
                r4
                \times 2/3
                {
                    <g b>2
                    r4
                }
                \time 3/4
                <d a>4
                - \tenuto
                r8
                <f a bf>4.
            }
        >>
        """
    )


def test_staff_splitter_13():
    staff = abjad.Staff(r"a4 b4 c'4 d'4")
    staves = abjad.staff_splitter(staff)
    score = abjad.Score(staves)
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                \clef "treble"
                r2
                c'4
                d'4
            }
            \new Staff
            {
                \clef "bass"
                a4
                b4
                r2
            }
        >>
        """
    )



import random

import abjad
import pytest

import auxjad


def test_Hocketer_01():
    random.seed(82132)
    container = abjad.Container(r"c'4 d'4 e'4 f'4")
    hocketer = auxjad.Hocketer(container)
    assert hocketer.n_voices == 2
    assert hocketer.weights == [1.0, 1.0]
    assert hocketer.k == 1
    assert abjad.lilypond(hocketer) == abjad.String.normalize(
        r"""
        {
            c'4
            d'4
            e'4
            f'4
        }
        """
    )
    music = hocketer()
    score = abjad.Score()
    for selection in music:
        score.append(abjad.Staff(selection))
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                r2
                e'4
                r4
            }
            \new Staff
            {
                c'4
                d'4
                r4
                f'4
            }
        >>
        """
    )
    music = hocketer.current_window
    with pytest.raises(AttributeError):
        hocketer.current_window = abjad.Container(r"c''2 e''2")
    score = abjad.Score()
    for selection in music:
        score.append(abjad.Staff(selection))
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                r2
                e'4
                r4
            }
            \new Staff
            {
                c'4
                d'4
                r4
                f'4
            }
        >>
        """
    )


def test_Hocketer_02():
    random.seed(48765)
    container = abjad.Container(r"c'8 d'8 e'8 f'8 g'8 a'8 b'8 c''8")
    hocketer = auxjad.Hocketer(container, n_voices=3)
    music = hocketer()
    score = abjad.Score()
    for selection in music:
        score.append(abjad.Staff(selection))
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                c'8
                r8
                e'8
                r8
                g'8
                r4.
            }
            \new Staff
            {
                r8
                d'8
                r4
                r8
                a'8
                r4
            }
            \new Staff
            {
                r4.
                f'8
                r4
                b'8
                c''8
            }
        >>
        """
    )


def test_Hocketer_03():
    container = abjad.Container(r"\time 3/4 c'4 d'4 e'4 \time 2/4 f'4 g'4")
    hocketer = auxjad.Hocketer(container,
                               n_voices=3,
                               weights=[1, 2, 5],
                               k=2,
                               force_k_voices=True,
                               explode_chords=True,
                               pitch_ranges=[abjad.PitchRange("[C4, D6]"),
                                             abjad.PitchRange("[C2, A4]"),
                                             abjad.PitchRange("[C1, E3]"),
                                             ],
                               disable_rewrite_meter=True,
                               use_multimeasure_rests=False,
                               omit_time_signatures=True,
                               boundary_depth=0,
                               maximum_dot_count=1,
                               rewrite_tuplets=False,
                               )
    assert hocketer.n_voices == 3
    assert hocketer.weights == [1, 2, 5]
    assert hocketer.k == 2
    assert hocketer.force_k_voices
    assert hocketer.explode_chords
    assert hocketer.pitch_ranges == [abjad.PitchRange("[C4, D6]"),
                                     abjad.PitchRange("[C2, A4]"),
                                     abjad.PitchRange("[C1, E3]"),
                                     ]
    assert hocketer.disable_rewrite_meter
    assert not hocketer.use_multimeasure_rests
    assert hocketer.omit_time_signatures
    assert hocketer.boundary_depth == 0
    assert hocketer.maximum_dot_count == 1
    assert not hocketer.rewrite_tuplets
    hocketer.n_voices = 5
    hocketer.weights = [1, 1, 1, 2, 7]
    hocketer.k = 3
    hocketer.force_k_voices = False
    hocketer.explode_chords = False
    hocketer.pitch_ranges = None
    hocketer.disable_rewrite_meter = False
    hocketer.use_multimeasure_rests = True
    hocketer.omit_time_signatures = False
    hocketer.boundary_depth = 1
    hocketer.maximum_dot_count = 2
    hocketer.rewrite_tuplets = True
    assert hocketer.n_voices == 5
    assert hocketer.weights == [1, 1, 1, 2, 7]
    assert hocketer.k == 3
    assert not hocketer.force_k_voices
    assert not hocketer.explode_chords
    assert hocketer.pitch_ranges is None
    assert not hocketer.disable_rewrite_meter
    assert hocketer.use_multimeasure_rests
    assert not hocketer.omit_time_signatures
    assert hocketer.boundary_depth == 1
    assert hocketer.maximum_dot_count == 2
    assert hocketer.rewrite_tuplets


def test_Hocketer_04():
    container = abjad.Container(r"c'4 d'4 e'4 f'4")
    hocketer = auxjad.Hocketer(container)
    assert abjad.lilypond(hocketer.contents) == abjad.String.normalize(
        r"""
        {
            c'4
            d'4
            e'4
            f'4
        }
        """
    )
    hocketer()
    assert abjad.lilypond(hocketer.contents) == abjad.String.normalize(
        r"""
        {
            c'4
            d'4
            e'4
            f'4
        }
        """
    )
    hocketer.contents = abjad.Container(r"cs2 ds2")
    assert abjad.lilypond(hocketer.contents) == abjad.String.normalize(
        r"""
        {
            cs2
            ds2
        }
        """
    )


def test_Hocketer_05():
    container = abjad.Container(r"c'2 d'2 e'2 f' 2")
    hocketer = auxjad.Hocketer(container)
    assert len(hocketer) == 2
    hocketer = auxjad.Hocketer(container, n_voices=7)
    assert len(hocketer) == 7


def test_Hocketer_06():
    random.seed(12174)
    container = abjad.Container(r"c'8 d'8 e'8 f'8 g'8 a'8 b'8 c''8")
    hocketer = auxjad.Hocketer(container)
    music = hocketer()
    score = abjad.Score()
    for selection in music:
        score.append(abjad.Staff(selection))
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                c'8
                d'8
                r4
                r8
                a'8
                r4
            }
            \new Staff
            {
                r4
                e'8
                f'8
                g'8
                r8
                b'8
                c''8
            }
        >>
        """
    )
    random.seed(12174)
    container = abjad.Container(r"c'8 d'8 e'8 f'8 g'8 a'8 b'8 c''8")
    hocketer = auxjad.Hocketer(container,
                               disable_rewrite_meter=True,
                               )
    music = hocketer()
    score = abjad.Score()
    for selection in music:
        score.append(abjad.Staff(selection))
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                c'8
                d'8
                r8
                r8
                r8
                a'8
                r8
                r8
            }
            \new Staff
            {
                r8
                r8
                e'8
                f'8
                g'8
                r8
                b'8
                c''8
            }
        >>
        """
    )


def test_Hocketer_07():
    random.seed(87201)
    container = abjad.Container(r"c'8 d'8 e'8 f'8 g'8 a'8 b'8 c''8")
    hocketer = auxjad.Hocketer(container, weights=[2.1, 5.7])
    music = hocketer()
    score = abjad.Score()
    for selection in music:
        score.append(abjad.Staff(selection))
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                r8
                d'8
                r4
                g'8
                r4.
            }
            \new Staff
            {
                c'8
                r8
                e'8
                f'8
                r8
                a'8
                b'8
                c''8
            }
        >>
        """
    )
    assert hocketer.weights == [2.1, 5.7]
    hocketer.reset_weights()
    assert hocketer.weights == [1.0, 1.0]


def test_Hocketer_08():
    random.seed(98212)
    container = abjad.Container(r"c'4 d'4 e'4 f'4")
    hocketer = auxjad.Hocketer(container, n_voices=4, k=2)
    music = hocketer()
    score = abjad.Score()
    for selection in music:
        score.append(abjad.Staff(selection))
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                c'4
                d'4
                e'4
                r4
            }
            \new Staff
            {
                r2
                e'4
                f'4
            }
            \new Staff
            {
                r2.
                f'4
            }
            \new Staff
            {
                r4
                d'4
                r2
            }
        >>
        """
    )


def test_Hocketer_09():
    random.seed(15663)
    container = abjad.Container(r"\time 3/4 c'4 d'4 e'4 f'4 g'4 a'4")
    hocketer = auxjad.Hocketer(container)
    music = hocketer()
    score = abjad.Score()
    for selection in music:
        score.append(abjad.Staff(selection))
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                \time 3/4
                R1 * 3/4
                r4
                g'4
                a'4
            }
            \new Staff
            {
                \time 3/4
                c'4
                d'4
                e'4
                f'4
                r2
            }
        >>
        """
    )
    random.seed(15663)
    container = abjad.Container(r"\time 3/4 c'4 d'4 e'4 f'4 g'4 a'4")
    hocketer = auxjad.Hocketer(container,
                               use_multimeasure_rests=False,
                               )
    music = hocketer()
    score = abjad.Score()
    for selection in music:
        score.append(abjad.Staff(selection))
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                \time 3/4
                r2.
                r4
                g'4
                a'4
            }
            \new Staff
            {
                \time 3/4
                c'4
                d'4
                e'4
                f'4
                r2
            }
        >>
        """
    )


def test_Hocketer_10():
    random.seed(14432)
    container = abjad.Container(r"c'8 d'8 e'8 f'8 g'8 a'8 b'8 c''8")
    hocketer = auxjad.Hocketer(container,
                               n_voices=3,
                               k=2,
                               force_k_voices=True,
                               )
    music = hocketer()
    score = abjad.Score()
    for selection in music:
        score.append(abjad.Staff(selection))
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                c'8
                d'8
                r8
                f'8
                g'8
                a'8
                b'8
                c''8
            }
            \new Staff
            {
                c'8
                r8
                e'8
                f'8
                r4
                b'8
                r8
            }
            \new Staff
            {
                r8
                d'8
                e'8
                r8
                g'8
                a'8
                r8
                c''8
            }
        >>
        """
    )


def test_Hocketer_11():
    container = abjad.Container(r"c'8 d'8 e'8 f'8 g'8 a'8 b'8 c''8")
    with pytest.raises(ValueError):
        hocketer = auxjad.Hocketer(container,  # noqa: F841
                                   n_voices=2,
                                   k=4,
                                   force_k_voices=True,
                                   )

    hocketer = auxjad.Hocketer(container,
                               n_voices=4,
                               k=3,
                               force_k_voices=True,
                               )
    with pytest.raises(ValueError):
        hocketer.k = 5
        hocketer.n_voices = 2

    hocketer = auxjad.Hocketer(container, n_voices=4, k=5)
    with pytest.raises(ValueError):
        hocketer.force_k_voices = True


def test_Hocketer_12():
    random.seed(48765)
    container = abjad.Container(
        r"\time 5/4 r4 \times 2/3 {c'4 d'2} e'4. f'8"
        r"\times 4/5 {\time 4/4 g'2. \times 2/3 {a'8 r8 b'2}}"
    )
    hocketer = auxjad.Hocketer(container,
                               n_voices=4,
                               k=2,
                               )
    music = hocketer()
    score = abjad.Score()
    for selection in music:
        score.append(abjad.Staff(selection))
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                \time 5/4
                r4
                \times 2/3
                {
                    c'4
                    d'2
                }
                r4.
                f'8
                \times 4/5
                {
                    \time 4/4
                    r2.
                    \times 2/3
                    {
                        a'8
                        r8
                        r2
                    }
                }
            }
            \new Staff
            {
                \time 5/4
                R1 * 5/4
                \time 4/4
                R1
            }
            \new Staff
            {
                \time 5/4
                r4
                \times 2/3
                {
                    r4
                    d'2
                }
                e'4.
                f'8
                \times 4/5
                {
                    \time 4/4
                    r2.
                    \times 2/3
                    {
                        a'8
                        r8
                        b'2
                    }
                }
            }
            \new Staff
            {
                \time 5/4
                r4
                \times 2/3
                {
                    c'4
                    r2
                }
                r2
                \times 4/5
                {
                    \time 4/4
                    g'2.
                    \times 2/3
                    {
                        r4
                        b'2
                    }
                }
            }
        >>
        """
    )


def test_Hocketer_13():
    container = abjad.Container(r"c'4. d'8 e'2")
    hocketer = auxjad.Hocketer(container,
                               n_voices=1,
                               )
    music = hocketer()
    score = abjad.Score()
    for selection in music:
        score.append(abjad.Staff(selection))
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                c'4.
                d'8
                e'2
            }
        >>
        """
    )
    hocketer = auxjad.Hocketer(container,
                               n_voices=1,
                               boundary_depth=1,
                               )
    music = hocketer()
    score = abjad.Score()
    for selection in music:
        score.append(abjad.Staff(selection))
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                c'4
                ~
                c'8
                d'8
                e'2
            }
        >>
        """
    )


def test_Hocketer_14():
    random.seed(15663)
    container = abjad.Container(r"\time 3/4 c'4 d'4 e'4 f'4 g'4 a'4")
    hocketer = auxjad.Hocketer(container,
                               omit_time_signatures=True,
                               use_multimeasure_rests=False,
                               )
    music = hocketer()
    score = abjad.Score()
    for selection in music:
        score.append(abjad.Staff(selection))
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                r2.
                r4
                g'4
                a'4
            }
            \new Staff
            {
                c'4
                d'4
                e'4
                f'4
                r2
            }
        >>
        """
    )


def test_Hocketer_15():
    random.seed(12432)
    container = abjad.Container(
        r"c'2-.\p\< d'2-.\f\> e'1 f'2.\pp\< g'4--\p "
        r"a'2\ff\> b'2\p\> ~ b'2 c''2\ppp"
    )
    hocketer = auxjad.Hocketer(container,
                               n_voices=3,
                               k=2,
                               force_k_voices=True,
                               )
    music = hocketer()
    score = abjad.Score()
    for selection in music:
        score.append(abjad.Staff(selection))
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                c'2
                \p
                - \staccato
                \<
                d'2
                \f
                - \staccato
                \>
                e'1
                f'2.
                \pp
                \<
                g'4
                \p
                - \tenuto
                a'2
                \ff
                \>
                r2
                \p
                r2
                c''2
                \ppp
            }
            \new Staff
            {
                r2
                d'2
                \f
                - \staccato
                \>
                e'1
                r2.
                \pp
                g'4
                \p
                - \tenuto
                r2
                b'2
                \>
                ~
                b'2
                r2
                \ppp
            }
            \new Staff
            {
                c'2
                \p
                - \staccato
                \<
                r2
                \f
                R1
                f'2.
                \pp
                \<
                r4
                \p
                a'2
                \ff
                \>
                b'2
                \p
                \>
                ~
                b'2
                c''2
                \ppp
            }
        >>
        """
    )


def test_Hocketer_16():
    container = abjad.Container(r"c'4 d'4 e'4 f'4")
    hocketer = auxjad.Hocketer(container)
    for voice in hocketer():
        assert isinstance(voice, abjad.Selection)
    tuplet = abjad.Tuplet('3:2', r"c'2 d'2 e'2")
    hocketer = auxjad.Hocketer(tuplet)
    for voice in hocketer():
        assert isinstance(voice, abjad.Selection)
    voice = abjad.Voice(r"c'4 d'4 e'4 f'4")
    hocketer = auxjad.Hocketer(voice)
    for voice in hocketer():
        assert isinstance(voice, abjad.Selection)
    staff = abjad.Staff(r"c'4 d'4 e'4 f'4")
    hocketer = auxjad.Hocketer(staff)
    for voice in hocketer():
        assert isinstance(voice, abjad.Selection)
    score = abjad.Score([abjad.Staff(r"c'4 d'4 e'4 f'4")])
    hocketer = auxjad.Hocketer(score)
    for voice in hocketer():
        assert isinstance(voice, abjad.Selection)
    voice = abjad.Voice(r"c'4 d'4 e'4 f'4")
    staff = abjad.Staff([voice])
    hocketer = auxjad.Hocketer(staff)
    for voice in hocketer():
        assert isinstance(voice, abjad.Selection)
    staff = abjad.Staff(r"c'4 d'4 e'4 f'4")
    score = abjad.Score([staff])
    hocketer = auxjad.Hocketer(score)
    for voice in hocketer():
        assert isinstance(voice, abjad.Selection)

    voice1 = abjad.Voice(r"c'4 d'4 e'4 f'4")
    voice2 = abjad.Voice(r"g2 f2")
    staff = abjad.Staff([voice1, voice2], simultaneous=True)
    with pytest.raises(ValueError):
        hocketer = auxjad.Hocketer(staff)  # noqa: F841

    staff1 = abjad.Staff(r"c'4 d'4 e'4 f'4")
    staff2 = abjad.Staff(r"g2 f2")
    score = abjad.Score([staff1, staff2])
    with pytest.raises(ValueError):
        hocketer = auxjad.Hocketer(score)  # noqa: F841


def test_Hocketer_17():
    random.seed(19876)
    container = abjad.Container(r"c'4 d'4 e'4 f'4")
    hocketer = auxjad.Hocketer(container, n_voices=5, k=3)
    hocketer()
    score = abjad.Score()
    for selection in hocketer[:]:
        staff = abjad.Staff(selection)
        score.append(staff)
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                r2.
                f'4
            }
            \new Staff
            {
                c'4
                r4
                e'4
                r4
            }
            \new Staff
            {
                r4
                d'4
                e'4
                r4
            }
            \new Staff
            {
                c'4
                d'4
                r4
                f'4
            }
            \new Staff
            {
                c'4
                r4
                e'4
                r4
            }
        >>
        """
    )
    staff = abjad.Staff(hocketer[0])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            r2.
            f'4
        }
        """
    )
    partial_score = abjad.Score()
    for selection in hocketer[1:4]:
        staff = abjad.Staff(selection)
        partial_score.append(staff)
    assert abjad.lilypond(partial_score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                c'4
                r4
                e'4
                r4
            }
            \new Staff
            {
                r4
                d'4
                e'4
                r4
            }
            \new Staff
            {
                c'4
                d'4
                r4
                f'4
            }
        >>
        """
    )


def test_Hocketer_18():
    random.seed(97813)
    container = abjad.Container(r"c4 d8 e'8 f'4 g'8 a8")
    hocketer = auxjad.Hocketer(container,
                               n_voices=2,
                               pitch_ranges=[abjad.PitchRange("[C4, B5]"),
                                             abjad.PitchRange("[C3, B3]"),
                                             ])
    hocketer()
    score = abjad.Score()
    for selection in hocketer[:]:
        staff = abjad.Staff(selection)
        score.append(staff)
    abjad.attach(abjad.Clef('bass'), abjad.select(score[1]).leaf(0))
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                r4.
                e'8
                f'4
                g'8
                r8
            }
            \new Staff
            {
                \clef "bass"
                c4
                d8
                r8
                r4.
                a8
            }
        >>
        """
    )


def test_Hocketer_19():
    random.seed(66191)
    container = abjad.Container(r"c' d' e f' g a' b' c''")
    hocketer = auxjad.Hocketer(container,
                               n_voices=3,
                               pitch_ranges=[abjad.PitchRange("[C4, C5]"),
                                             abjad.PitchRange("[C4, C5]"),
                                             abjad.PitchRange("[C3, B3]"),
                                             ])
    hocketer()
    score = abjad.Score()
    for selection in hocketer[:]:
        staff = abjad.Staff(selection)
        score.append(staff)
    abjad.attach(abjad.Clef('bass'), abjad.select(score[2]).leaf(0))
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                c'4
                d'4
                r4
                f'4
                r4
                a'4
                r2
            }
            \new Staff
            {
                R1
                r2
                b'4
                c''4
            }
            \new Staff
            {
                \clef "bass"
                r2
                e4
                r4
                g4
                r2.
            }
        >>
        """
    )


def test_Hocketer_20():
    random.seed(81742)
    container = abjad.Container(r"c'8 d'8 e8 f'8 g8 a'8 b'8 c''8")
    hocketer = auxjad.Hocketer(container,
                               n_voices=4,
                               k=2,
                               force_k_voices=True,
                               pitch_ranges=[abjad.PitchRange("[C4, C5]"),
                                             abjad.PitchRange("[C4, C5]"),
                                             abjad.PitchRange("[E3, G4]"),
                                             abjad.PitchRange("[E3, G4]"),
                                             ])
    hocketer()
    score = abjad.Score()
    for selection in hocketer[:]:
        staff = abjad.Staff(selection)
        score.append(staff)
    abjad.attach(abjad.Clef('bass'), abjad.select(score[2]).leaf(0))
    abjad.attach(abjad.Clef('bass'), abjad.select(score[3]).leaf(0))
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                r2
                r8
                a'8
                b'8
                c''8
            }
            \new Staff
            {
                c'8
                d'8
                r4
                r8
                a'8
                b'8
                c''8
            }
            \new Staff
            {
                \clef "bass"
                r8
                d'8
                e8
                f'8
                g8
                r4.
            }
            \new Staff
            {
                \clef "bass"
                c'8
                r8
                e8
                f'8
                g8
                r4.
            }
        >>
        """
    )


def test_Hocketer_21():
    random.seed(48124)
    container = abjad.Container(
        r"<c' e' g'>4 <d' f' a'>4 <e' g' b'>4 <f' a' c'>4"
    )
    hocketer = auxjad.Hocketer(container,
                               n_voices=3,
                               )
    hocketer()
    score = abjad.Score()
    for selection in hocketer[:]:
        staff = abjad.Staff(selection)
        score.append(staff)
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                r2
                <e' g' b'>4
                r4
            }
            \new Staff
            {
                <c' e' g'>4
                <d' f' a'>4
                r2
            }
            \new Staff
            {
                r2.
                <c' f' a'>4
            }
        >>
        """
    )

    container = abjad.Container(
        r"<c' e' g'>4 <d' f' a'>4 <e' g' b'>4 <f' a' c'>4"
    )
    hocketer = auxjad.Hocketer(container,
                               n_voices=3,
                               explode_chords=True,
                               )
    hocketer()
    score = abjad.Score()
    for selection in hocketer[:]:
        staff = abjad.Staff(selection)
        score.append(staff)
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                e'4
                f'4
                e'4
                a'4
            }
            \new Staff
            {
                g'4
                d'4
                b'4
                c'4
            }
            \new Staff
            {
                c'4
                a'4
                g'4
                f'4
            }
        >>
        """
    )


def test_Hocketer_22():
    random.seed(91776)
    container = abjad.Container(
        r"<c' e' g'>4 <d' f' a'>4 <e' g' b'>4 <f' a' c''>4"
    )
    hocketer = auxjad.Hocketer(container,
                               n_voices=3,
                               explode_chords=True,
                               pitch_ranges=[abjad.PitchRange("[E4, C5]"),
                                             abjad.PitchRange("[E4, C5]"),
                                             abjad.PitchRange("[C4, F4]"),
                                             ],
                               )
    hocketer()
    score = abjad.Score()
    for selection in hocketer[:]:
        staff = abjad.Staff(selection)
        score.append(staff)
    assert abjad.lilypond(score) == abjad.String.normalize(
        r"""
        \new Score
        <<
            \new Staff
            {
                e'4
                f'4
                g'4
                c''4
            }
            \new Staff
            {
                g'4
                a'4
                b'4
                a'4
            }
            \new Staff
            {
                c'4
                d'4
                e'4
                f'4
            }
        >>
        """
    )


def test_Hocketer_23():
    container = abjad.Container(r"\time 3/4 c'4 d'4 e'4 \time 2/4 f'4 g'4")
    hocketer = auxjad.Hocketer(container,
                               n_voices=3,
                               pitch_ranges=[abjad.PitchRange("[C4, D6]"),
                                             abjad.PitchRange("[C2, A4]"),
                                             abjad.PitchRange("[C1, E3]"),
                                             ],
                               )
    assert hocketer.pitch_ranges == [abjad.PitchRange("[C4, D6]"),
                                     abjad.PitchRange("[C2, A4]"),
                                     abjad.PitchRange("[C1, E3]"),
                                     ]
    hocketer.n_voices = 4
    assert hocketer.pitch_ranges is None



import abjad

import auxjad


def test_piano_pedal_01():
    staff = abjad.Staff(r"c'4 d'4 e'4 f'4")
    auxjad.piano_pedal(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'4
            \sustainOn
            d'4
            e'4
            f'4
            \sustainOff
        }
        """
    )


def test_piano_pedal_02():
    staff = abjad.Staff(r"c'4 d'4 e'4 f'4")
    auxjad.piano_pedal(staff[:],
                       until_the_end=True,
                       )
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \once \override Staff.SustainPedal.stencil =
                #(lambda (grob) (grob-interpret-markup grob
                    #{
                        \markup {
                            \concat {
                                \musicglyph "pedal.Ped"
                                \musicglyph "pedal.."
                            }
                            \raise #-0.3 "→"
                        }
                    #}))
            c'4
            \sustainOn
            d'4
            e'4
            f'4
            \sustainOff
        }
        """
    )


def test_piano_pedal_03():
    staff = abjad.Staff(r"c'4 d'4 e'4 f'4")
    auxjad.piano_pedal(staff[:],
                       omit_raise_pedal_glyph=True,
                       )
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'4
            \sustainOn
            d'4
            e'4
            \once \override Staff.SustainPedal.stencil = ##f
            f'4
            \sustainOff
        }
        """
    )


def test_piano_pedal_04():
    staff = abjad.Staff(r"c'4 d'4 e'4 f'4")
    auxjad.piano_pedal(staff[:],
                       until_the_end=True,
                       omit_raise_pedal_glyph=True,
                       )
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \once \override Staff.SustainPedal.stencil =
                #(lambda (grob) (grob-interpret-markup grob
                    #{
                        \markup {
                            \concat {
                                \musicglyph "pedal.Ped"
                                \musicglyph "pedal.."
                            }
                            \raise #-0.3 "→"
                        }
                    #}))
            c'4
            \sustainOn
            d'4
            e'4
            \once \override Staff.SustainPedal.stencil = ##f
            f'4
            \sustainOff
        }
        """
    )


def test_piano_pedal_05():
    staff = abjad.Staff(r"c'4 d'4 e'4 f'4")
    auxjad.piano_pedal(staff[:])
    abjad.setting(staff).pedal_sustain_style = "#'mixed"
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        \with
        {
            pedalSustainStyle = #'mixed
        }
        {
            c'4
            \sustainOn
            d'4
            e'4
            f'4
            \sustainOff
        }
        """
    )


def test_piano_pedal_06():
    staff = abjad.Staff(r"c'4 d'4 e'4 f'4")
    auxjad.piano_pedal(staff[:],
                       until_the_end=True,
                       )
    abjad.setting(staff).pedal_sustain_style = "#'mixed"
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        \with
        {
            pedalSustainStyle = #'mixed
        }
        {
            \once \override Staff.SustainPedal.stencil =
                #(lambda (grob) (grob-interpret-markup grob
                    #{
                        \markup {
                            \concat {
                                \musicglyph "pedal.Ped"
                                \musicglyph "pedal.."
                            }
                            \raise #-0.3 "→"
                        }
                    #}))
            c'4
            \sustainOn
            d'4
            e'4
            f'4
            \sustainOff
        }
        """
    )


def test_piano_pedal_07():
    staff = abjad.Staff(r"c'4 d'4 e'4 f'4")
    auxjad.piano_pedal(staff[:],
                       until_the_end=True,
                       omit_raise_pedal_glyph=True,
                       )
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \once \override Staff.SustainPedal.stencil =
                #(lambda (grob) (grob-interpret-markup grob
                    #{
                        \markup {
                            \concat {
                                \musicglyph "pedal.Ped"
                                \musicglyph "pedal.."
                            }
                            \raise #-0.3 "→"
                        }
                    #}))
            c'4
            \sustainOn
            d'4
            e'4
            \once \override Staff.SustainPedal.stencil = ##f
            f'4
            \sustainOff
        }
        """
    )


def test_piano_pedal_08():
    staff = abjad.Staff(r"c'4 d'4 e'4 f'4")
    auxjad.piano_pedal(staff[:],
                       half_pedal=True,
                       )
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \once \override Staff.SustainPedal.stencil =
                #(lambda (grob) (grob-interpret-markup grob
                    #{
                        \markup {
                            \larger "½"
                            \concat {
                                \musicglyph "pedal.Ped"
                                \musicglyph "pedal.."
                            }
                        }
                    #}))
            c'4
            \sustainOn
            d'4
            e'4
            f'4
            \sustainOff
        }
        """
    )


def test_piano_pedal_09():
    staff = abjad.Staff(r"c'4 d'4 e'4 f'4")
    auxjad.piano_pedal(staff[:],
                       half_pedal=True,
                       until_the_end=True,
                       )
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \once \override Staff.SustainPedal.stencil =
                #(lambda (grob) (grob-interpret-markup grob
                    #{
                        \markup {
                            \larger "½"
                            \concat {
                                \musicglyph "pedal.Ped"
                                \musicglyph "pedal.."
                            }
                            \raise #-0.3 "→"
                        }
                    #}))
            c'4
            \sustainOn
            d'4
            e'4
            f'4
            \sustainOff
        }
        """
    )


def test_piano_pedal_10():
    staff = abjad.Staff(r"c'4 d'4 e'4 f'4")
    auxjad.piano_pedal(staff[:],
                       half_pedal=True,
                       omit_raise_pedal_glyph=True,
                       )
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \once \override Staff.SustainPedal.stencil =
                #(lambda (grob) (grob-interpret-markup grob
                    #{
                        \markup {
                            \larger "½"
                            \concat {
                                \musicglyph "pedal.Ped"
                                \musicglyph "pedal.."
                            }
                        }
                    #}))
            c'4
            \sustainOn
            d'4
            e'4
            \once \override Staff.SustainPedal.stencil = ##f
            f'4
            \sustainOff
        }
        """
    )


def test_piano_pedal_11():
    staff = abjad.Staff(r"c'4 d'4 e'4 f'4")
    auxjad.piano_pedal(staff[:],
                       half_pedal=True,
                       until_the_end=True,
                       omit_raise_pedal_glyph=True,
                       )
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \once \override Staff.SustainPedal.stencil =
                #(lambda (grob) (grob-interpret-markup grob
                    #{
                        \markup {
                            \larger "½"
                            \concat {
                                \musicglyph "pedal.Ped"
                                \musicglyph "pedal.."
                            }
                            \raise #-0.3 "→"
                        }
                    #}))
            c'4
            \sustainOn
            d'4
            e'4
            \once \override Staff.SustainPedal.stencil = ##f
            f'4
            \sustainOff
        }
        """
    )


def test_piano_pedal_12():
    staff = abjad.Staff(r"c'4 d'4 e'4 f'4")
    auxjad.piano_pedal(staff[:],
                       half_pedal=True,
                       )
    abjad.setting(staff).pedal_sustain_style = "#'mixed"
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        \with
        {
            pedalSustainStyle = #'mixed
        }
        {
            \once \override Staff.SustainPedal.stencil =
                #(lambda (grob) (grob-interpret-markup grob
                    #{
                        \markup {
                            \larger "½"
                            \concat {
                                \musicglyph "pedal.Ped"
                                \musicglyph "pedal.."
                            }
                        }
                    #}))
            c'4
            \sustainOn
            d'4
            e'4
            f'4
            \sustainOff
        }
        """
    )


def test_piano_pedal_13():
    staff = abjad.Staff(r"c'4 d'4 e'4 f'4")
    auxjad.piano_pedal(staff[:],
                       half_pedal=True,
                       until_the_end=True,
                       )
    abjad.setting(staff).pedal_sustain_style = "#'mixed"
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        \with
        {
            pedalSustainStyle = #'mixed
        }
        {
            \once \override Staff.SustainPedal.stencil =
                #(lambda (grob) (grob-interpret-markup grob
                    #{
                        \markup {
                            \larger "½"
                            \concat {
                                \musicglyph "pedal.Ped"
                                \musicglyph "pedal.."
                            }
                            \raise #-0.3 "→"
                        }
                    #}))
            c'4
            \sustainOn
            d'4
            e'4
            f'4
            \sustainOff
        }
        """
    )


def test_piano_pedal_14():
    staff = abjad.Staff(r"c'4 d'4 e'4 f'4")
    auxjad.piano_pedal(staff[:],
                       half_pedal=True,
                       until_the_end=True,
                       omit_raise_pedal_glyph=True,
                       )
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \once \override Staff.SustainPedal.stencil =
                #(lambda (grob) (grob-interpret-markup grob
                    #{
                        \markup {
                            \larger "½"
                            \concat {
                                \musicglyph "pedal.Ped"
                                \musicglyph "pedal.."
                            }
                            \raise #-0.3 "→"
                        }
                    #}))
            c'4
            \sustainOn
            d'4
            e'4
            \once \override Staff.SustainPedal.stencil = ##f
            f'4
            \sustainOff
        }
        """
    )


def test_piano_pedal_15():
    staff = abjad.Staff(r"c'4 d'4 e'4 f'4")
    abjad.piano_pedal(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'4
            \sustainOn
            d'4
            e'4
            f'4
            \sustainOff
        }
        """
    )
